<?php

namespace SM\Service;

use Doctrine\ORM\EntityManager;
use Teknisa\Libs\Util\Environment;
use SM\Factory\AssinaturaDigitalCDAFactory as AssinaturaFactory;
use Teknisa\Libs\Util\SMTP as SMTPService;
use SM\Util\Utilities;
use Teknisa\Libs\Util\ConnectionCustom;
use Teknisa\Libs\Util\NewCode;
//require_once "/home/developer/workfolder/sm/backend/vendor/pear/http_request2/HTTP/Request2.php";
//require_once "HTTP/Request2.php";
require_once ('../vendor/pear/http_request2/HTTP/Request2.php');

class AssinaturaDigitalCDAService {

    protected $entityManager;
    protected $connection;
    protected $environment;
    /** @var AssinaturaFactory */
    protected $assinaturaFactory;
    protected $urlIntegracaoCertisign;
    protected $smtpService;
    protected $connectionCustom;
    protected $newCode;
    

    public function __construct(EntityManager $entityManager, Environment $environment, $urlIntegracaoCertisign, ConnectionCustom $connectionCustom) {
        $this->entityManager          = $entityManager;
        $this->environment            = $environment;
        $this->urlIntegracaoCertisign = $urlIntegracaoCertisign;
        $this->connectionCustom       = $connectionCustom;
        
        $this->iniciarDependencias(null);
    }
    
    private function iniciarDependencias($nrorg) {
        if (is_null($nrorg))
            $nrorg = $this->environment->getNrOrg();
            
        $this->setConnection($nrorg);
        $this->assinaturaFactory = new AssinaturaFactory($this->entityManager);
        $this->newCode = new NewCode($this->connection, "NOVO_CODIGO_SC");
    }
    
    private function setConnection($nrorg) {
        $novoEntityManager = $this->connectionCustom->updateVpdConnection($nrorg);
        if ($novoEntityManager) {
            $this->entityManager = $novoEntityManager;
            $this->connection    = $novoEntityManager->getConnection();
        }
        else {
            $this->connection = $this->entityManager->getConnection();
        }
    }

    public function convertPDFinBase64($documento){
        try {

            $handle = fopen($documento,'r');
            $file_content = fread($handle,filesize($documento));
            fclose($handle);

            // $byte_array = unpack("C*", $file_content);

            $encoded = base64_encode($file_content);

            return $encoded;

        } catch(\Exception $e){
            throw $e;
        }
    }

    public function buildJsontoUpload($nmDocumento, $documento){
        
        $base64 = $this->convertPDFinBase64($documento);

        $arrayupload = array(
            "fileName" => $nmDocumento,
            "bytes"  => $base64
        );

        $jsonUpload = json_encode($arrayupload, JSON_PRETTY_PRINT);

        return $jsonUpload;
    }

    public function uploadtoCertsing($nmDocumento, $documento){
        
        $token = $this->getTokenCertsign();

        $return = array();
        
        $request = new \Http_Request2($this->urlIntegracaoCertisign.'document/upload', \Http_Request2::METHOD_POST);
        $url = $request->getUrl();

        $headers = array(
            // Request headers
            'Content-Type' => 'application/json',
            'Token' => $token,
        );

        $request->setHeader($headers);

        $parameters = array(
            // Request parameters
        );

        $url->setQueryVariables($parameters);

        $request->setMethod(\Http_Request2::METHOD_POST);

        $body = $this->buildJsontoUpload($nmDocumento, $documento);

        // Request body
        $request->setBody($body);
        
        $return['request']['parameters'] = $parameters;
        $return['request']['body'] = json_decode($body, true);

        try {
            $response = $request->send();
            
            $retorno = $response->getBody();
            $retorno = json_decode($retorno, true);
            
            $return['response'] = $retorno;
            return $return;
        }
        catch (\HTTP_Request2_Exception $ex) {
            //echo $ex;
            $return['response'] = $ex;
            return $return;
        }
    }

    public function buildJsontoCreate($uploadId, $nmDocumento, $sender, $eletrSigners, $padraoAssinatura, $tipoDoc, $emailOperResp){

        /* -- Documento -- */ 
        $document = array(
            "name" => $nmDocumento,
            "upload" => array("id" => $uploadId,
                              "name" => $nmDocumento)
            );

        /* -- responsável pelo envio -- */
        $sender = array(
            "name"  => $sender['NOME'],
            "email" => $sender['EMAIL'],
            "individualIdentificationCode" => $sender['CPF']
        );
        
        /* -- Signatários Digitais -- */
       
        $signers_aux = array();

        /*  Por hora, não possuem signatários digitais. Caso um dia seja habilitado essa função, deve ser 
             colocado entre os parâmetros da função, a variável "$signers" e descomentado este pedaço. **
 
        foreach ($signers as $ind => $signer) {
        
            $signers_aux[$ind] = array(
                "step"  => $signer['POSICAO'],
                "title" => $signer['TIPO'],
                "name"  => $signer['NOME'],
                "email" => $signer['EMAIL'],
                //"email" => 'ranzobh@yahoo.com.br',
                "individualIdentificationCode"=> $signer['CPF']
            );
        }  */

        /* -- Signatários Eletrônicos -- */
        
        $eletrSigners_aux = array();
        
        foreach ($eletrSigners as $ind => $eletr) {
        
            $eletrSigners_aux[$ind] = array(
                "step"  => $eletr['POSICAO'],
                "title" => $eletr['TIPO'],
                "name"  => $eletr['NOME'],
                "email" => $eletr['EMAIL'],
                //"email" => 'ranzobh@yahoo.com.br',
                "individualIdentificationCode"=> $eletr['CPF']
            );
        }
        
        /* -- Cria e envia o body do documento -- */

        $arrayCreate = array(
            "document" => $document,
            "sender"  => $sender,
            "signers" => $signers_aux,
            "electronicSigners" => $eletrSigners_aux
        );
        
        if($padraoAssinatura){
            $arrayCreate["signatureStandard"] = $padraoAssinatura;
        }
        if($tipoDoc){
            /* Descomentar a linha abaixo e comentar a linha com '$tipoDoc' para teste da rotina em ambiente de homologação. */
            // $arrayCreate["typeId"] = '2889'; 
            $arrayCreate["typeId"] =  $tipoDoc;
        }
        if($emailOperResp){
            $arrayCreate["recipients"] = array(
                "emails" => $emailOperResp,
                "attached" => true
            );
        }
        
        $jsonCreate = json_encode($arrayCreate, JSON_PRETTY_PRINT);
        
        return $jsonCreate;
    }

    public function createSignatureCertsing($uploadId, $nmDocumento, $sender, $signers, $eletrSigners, $padraoAssinatura, $tipoDoc, $emailOperResp){
        
        $token = $this->getTokenCertsign();
        
        $return = array();

        try {
            $request = new \Http_Request2($this->urlIntegracaoCertisign.'document/create', \Http_Request2::METHOD_POST);
            $url = $request->getUrl();
    
            $headers = array(
                // Request headers
                'Content-Type' => 'application/json',
                'Token' => $token,
            );
    
            $request->setHeader($headers);
    
            $parameters = array(
                // Request parameters
            );
    
            $url->setQueryVariables($parameters);
    
            $request->setMethod(\Http_Request2::METHOD_POST);
    
            $body = $this->buildJsontoCreate($uploadId, $nmDocumento, $sender, $eletrSigners, $padraoAssinatura, $tipoDoc, $emailOperResp);
            $body = str_replace('\/', '/', $body);
            
            $return['request']['parameters'] = $parameters;
            $return['request']['body'] = json_decode($body, true);
            // Request body
            $request->setBody($body);
             
            try {
                $response = $request->send();
                
                $retorno = $response->getBody();
                $retorno = json_decode($retorno, true);
                
                $return['response'] = $retorno;
                
                return $return;
            }
            catch (\HTTP_Request2_Exception $ex)
            {
                $return['response'] = $ex;
    
                return $return;
            }

        } catch(\Exception $e){
            throw $e;
        }
    }

    public function callbackCertsing($filedata, $nrorg) {
        $this->iniciarDependencias($nrorg);
        $assinouOuRecusou = null;
        
        try {
            $action = $filedata['action'];
            
            if (($action == 'SIGNATURE-ELETRONIC') || ($action == 'SIGNATURE-DIGITAL')) {
                $assinouOuRecusou = 'A';
                
                $documentId = $filedata['documentId'];
                $identifier = $filedata['identifier'];
                
                $params = array(
                    "IDDOCUMCERTSIGN" => $documentId
                );
                $getCDAs = $this->assinaturaFactory->getServPrestDoc($params);
                
                if (count($getCDAs) > 0) {

                    foreach($getCDAs as $cda) {
                        $servPrestDoc = $cda['NRSERVPRESTDOC'];
                        
                        $params = array(
                            "NRSERVPRESTDOC"  => $servPrestDoc,
                            "NRCPFSIGNATARIO" => $identifier
                        );
                        
                        $seqSignatario = $this->assinaturaFactory->recebeSignatarioCDA($params);
                        
                        if (!empty($seqSignatario))
                            $signatario = $seqSignatario[0]["NRSERVPRESTDOCASSINA"];
                    
                        $params = array(
                            "IDSITASSINATURA"       => 'ASSINADO',
                            "DSJUSTIFRECUSAASSINAT" => null,
                            "NRSERVPRESTDOC"        => $servPrestDoc,
                            "NRSERVPRESTDOCASSINA"  => $signatario
                        );
                        
                        $this->assinaturaFactory->updateAssinaDocCDA($params);

                      //  $this->enviaEmailCertisign($identifier, $servPrestDoc, $tipo, $codigoPropCont, $nmarquivo, $dsmodelodocorca, $assinouOuRecusou, $nrorg, null, $documentId, $documento);
                    }
                    $this->verificaAceitesCallback($documentId);
                }
                else {
                    throw new \Exception("Não foi encontrado um documento com o ID informado.");
                }
            }
            else if ($action == 'SIGNATURE-NOK') {
                $assinouOuRecusou = 'R';
                
                $documentId = $filedata['documentId'];
                $identifier = $filedata['identifier'];
                $message    = $filedata['message'];
                
                $params = array(
                    "IDDOCUMCERTSIGN" => $documentId
                );
                
                $getCDAs = $this->assinaturaFactory->getServPrestDoc($params);
                
                if (count($getCDAs) > 0) {
                    foreach($getCDAs as $cda) {
                        $servPrestDoc = $cda['NRSERVPRESTDOC'];
                            
                        $params = array(
                            "NRSERVPRESTDOC"  => $servPrestDoc,
                            "NRCPFSIGNATARIO" => $identifier
                        );
                        
                        $seqSignatario = $this->assinaturaFactory->recebeSignatarioCDA($params);
                        
                        if (!empty($seqSignatario))
                            $signatario = $seqSignatario[0]["NRSERVPRESTDOCASSINA"];
                        
                        $params = array(
                            "IDSITASSINATURA"       => 'RECUSADO',
                            "DSJUSTIFRECUSAASSINAT" => $message,
                            "NRSERVPRESTDOC"        => $servPrestDoc,
                            "NRSERVPRESTDOCASSINA"  => $signatario
                        );
                        
                        $this->assinaturaFactory->updateAssinaDocCDA($params);

                        //$this->enviaEmailCertisign($identifier, $servPrestDoc, $tipo, $codigoPropCont, $nmarquivo, $dsmodelodocorca, $assinouOuRecusou, $nrorg, $message, $documentId, $documento);
                    }
                    $this->verificaAceitesCallback($documentId);
                }
                else {
                    throw new \Exception("Não foi encontrado um documento com o ID informado.");
                }
            }
        }
        catch(\Exception $e) {
            //file_put_contents("/home/developer/workfolder/TESTE/erro.txt", $e->getMessage());

            $iddocumcertsign = isset($documentId) ? $documentId : null;
            $nrcpfsignatario = isset($identifier) ? $identifier : null;
            $dserro = "Erro ao atualizar a situação do documento: " . $e->getMessage();
            
            $params = array(
                "IDDOCUMCERTSIGN" => $documentId
            );
            $getCDAs = $this->assinaturaFactory->getServPrestDoc($params);

            foreach($getCDAs as $cda) {

                $nrservprestdoc = $cda['NRSERVPRESTDOC'];

                $params = array(
                    "NRSERVPRESTDOC"  => $nrservprestdoc,
                    "NRCPFSIGNATARIO" => $identifier
                );

                $seqSignatario = $this->assinaturaFactory->recebeSignatarioCDA($params);
                        
                if (!empty($seqSignatario))
                    $nrservprestdocassina = $seqSignatario[0]["NRSERVPRESTDOCASSINA"];

                $nrservprestdoclogassina = $this->newCode->getCodeByTableName("FCT_LOGASSINATDIGITAL", $nrorg, 1, 10, true);
                
                $params = array(
                    "NRSERVPRESTDOCLOGASSINA" => $nrservprestdoclogassina,
                    "NRSERVPRESTDOC"          => $nrservprestdoc, 
                    "NRSERVPRESTDOCASSINA"    => $nrservprestdocassina,
                    "IDDOCUMCERTSIGN"         => $iddocumcertsign,
                    "NRCPFSIGNATARIO"         => $nrcpfsignatario,
                    "DSERRO"                  => $dserro,
                    "NRORG"                   => $nrorg
                );
                
                $this->assinaturaFactory->inserirLogAssinaturaDigitalCDA($params);
            }
            throw $e;
        }
    }
    
    /* Verifica se todos os as situações dos documentos de assinatura para atuzalizar os registros dos signatários. Sendo:
       - Se alguns dos documentos estiver recusado,  o signatário fica como recusado.
       - Fora o caso acima, caso ainda tenha algum documento pendente, o signatário permanece pendente.
       - Caso não haja nenhum pendente e nenhum recusado, o signatário fica aprovado.
       Obs: os dados de assinatura manual serão tratados na própria ação de asssinatura manual.
    */
    public function verificaAceitesCallback($documentId, $dataAssina = null) {
        try {
            $countAprovado  = 0;
            $countRecusado = 0;
            $countPendente  = 0;
            $justificaRecusa = null;
            
            $params = array(
                "IDDOCUMCERTSIGN" => $documentId
            );
            
            $getCDAs = $this->assinaturaFactory->getServPrestDoc($params);
            
            foreach($getCDAs as $cda) {
                $servPrestDoc = $cda['NRSERVPRESTDOC'];

                $params = array(
                    "NRSERVPRESTDOC"   => $servPrestDoc
                );
                
                $dadosSignatario = $this->assinaturaFactory->recebeDadosSignatariosCDA($params);  
                
                foreach ($dadosSignatario as $dados){
                    
                    if ($dados["IDSITASSINATURA"] == 'ASSINADO'){
                        
                        $countAprovado = $countAprovado+1;
                    } 
                    else if ($dados["IDSITASSINATURA"] == 'RECUSADO'){ 
                        
                        $countRecusado = $countRecusado+1;
                        
                        if ($justificaRecusa == null) {
                            
                            $justificaRecusa = $dados["DSJUSTIFRECUSAASSINAT"];
                        } 
                        else {
                            
                            $justificaRecusa = $justificaRecusa.'    /    '.$dados["DSJUSTIFRECUSAASSINAT"];
                        }
                    } 
                    else if ($dados["IDSITASSINATURA"] == 'ENVIADO'){
                        
                        $countPendente = $countPendente+1;
                    } 
                } 

                if ($countRecusado > 0) {
                    $params = array(
                        "IDSITASSINATURA"       => 'RECUSADO',
                        "DTASSINATURA"          => $dataAssina,
                        "DSJUSTIFRECUSAASSINAT" => $justificaRecusa,
                        "NRSERVPRESTDOC"        => $servPrestDoc
                    );
                    
                    $this->assinaturaFactory->updateServPrestDoc($params, 'R');
                }
                else if ($countPendente > 0) {
                    $params = array(
                        "IDSITASSINATURA"        => 'PARCIAL',
                        "NRSERVPRESTDOC"         => $servPrestDoc
                    );
                    
                    $this->assinaturaFactory->updateServPrestDoc($params, 'P');
                }
                else {
                    $params = array(
                        "IDSITASSINATURA"        => 'ASSINADO',
                        "DTASSINATURA"          => $dataAssina,
                        "NRSERVPRESTDOC"         => $servPrestDoc
                    );
                    
                    $this->assinaturaFactory->updateServPrestDoc($params, 'A');
                }
            }
            
        } catch(\Exception $e){
            throw $e;
        }
    }
    
    public function incluirSignatario($documentId, $signers, $eletrSigners) {
        
        $token = $this->getTokenCertsign();
        
        $return = array();

        try {
            $request = new \Http_Request2($this->urlIntegracaoCertisign.'document/participantAdd', \Http_Request2::METHOD_POST);
            $url = $request->getUrl();
    
            $headers = array(
                // Request headers
                'Content-Type' => 'application/json',
                'Token' => $token,
            );
    
            $request->setHeader($headers);
    
            $parameters = array(
                // Request parameters
            );
            
            $url->setQueryVariables($parameters);
            
            $request->setMethod(\Http_Request2::METHOD_POST);
            
            $body = $this->buildJsontoIncSignatario($documentId, $signers, $eletrSigners);
            
            $return['request']['parameters'] = $parameters;
            $return['request']['body'] = json_decode($body, true);
             
            // Request body
            $request->setBody($body);
        
            try {
                $response = $request->send();
                $retorno =  $response->getBody();
                $retorno =  json_decode($retorno, true);
                
                $return['response'] = $retorno;
                
                return $return;
            }
            catch (\HTTP_Request2_Exception $ex) {
                $return['response'] = $ex;
                return $return;
            }
        }
        catch(\Exception $e){
            throw $e;
        }
    }
    
    public function buildJsontoIncSignatario($documentId, $signers, $eletrSigners){

        /* -- Signatários Digitais -- */
        
        $signers_aux = array();
        
        foreach ($signers as $ind => $signer) {
        
            $signers_aux[$ind] = array(
                "step"  => $signer['POSICAO'],
                "title" => $signer['TIPO'],
                "name"  => $signer['NOME'],
                "email" => $signer['EMAIL'],
                //"email" => 'ranzobh@yahoo.com.br',
                "individualIdentificationCode"=> $signer['CPF']
            );
        }
        
        /* -- Signatários Eletrônicos -- */
        
        $eletrSigners_aux = array();
        
        foreach ($eletrSigners as $ind => $eletr) {
        
            $eletrSigners_aux[$ind] = array(
                "step"  => $eletr['POSICAO'],
                "title" => $eletr['TIPO'],
                "name"  => $eletr['NOME'],
                "email" => $eletr['EMAIL'],
                //"email" => 'ranzobh@yahoo.com.br',
                "individualIdentificationCode"=> $eletr['CPF']
            );
        }
        
        /* -- Cria e envia o body do documento -- */

        $arrayCreate = array(
            "documentId" => $documentId,
            "signers" => $signers_aux,
            "electronicSigners" => $eletrSigners_aux
        );
        
        $jsonCreate = json_encode($arrayCreate, JSON_PRETTY_PRINT);

        return $jsonCreate;
    }
    
    public function excluirSignatario($documentId, $idsignatcertsign, $nrcpfsignatario) {
        
        $token = $this->getTokenCertsign();
        
        $return = array();

        try {
            $request = new \Http_Request2($this->urlIntegracaoCertisign.'document/participantDiscard', \Http_Request2::METHOD_POST);
            $url = $request->getUrl();
    
            $headers = array(
                // Request headers
                'Content-Type' => 'application/json',
                'Token' => $token,
            );
            
            $request->setHeader($headers);
            
            $parameters = array(
                // Request parameters
            );
            
            $url->setQueryVariables($parameters);
            
            $request->setMethod(\Http_Request2::METHOD_POST);
            
            $bodyCreate = array(
                "documentId" => $documentId,
                "individualIdentificationCode" => $nrcpfsignatario
            );

            $body = json_encode($bodyCreate, JSON_PRETTY_PRINT);
            
            // Request body
            $request->setBody($body);
            
            $return['request']['parameters'] = $parameters;
            $return['request']['body'] = json_decode($body, true);
             
            try {
                $response = $request->send();
                
                $retorno =  $response->getBody();
                $retorno =  json_decode($retorno, true);
                
                $return['response'] = $retorno;
                
                return $return;
            }
            catch (\HTTP_Request2_Exception $ex) {
                $return['response'] = $ex;
                return $return;
            }
        }
        catch(\Exception $e){
            throw $e;
        }
    }
    
    public function reenviarSolicitacao($documentId) {
        
        $token = $this->getTokenCertsign();
        
        $return = array();

        try {
            $request = new \Http_Request2($this->urlIntegracaoCertisign.'document/SendReminder', \Http_Request2::METHOD_POST);
            $url = $request->getUrl();
    
            $headers = array(
                // Request headers
                'Content-Type' => 'application/json',
                'Token' => $token,
            );
            
            $request->setHeader($headers);
            
            $parameters = array(
                // Request parameters
            );
            
            $url->setQueryVariables($parameters);
            
            $request->setMethod(\Http_Request2::METHOD_POST);
            
            $bodyCreate = array(
                "documents" => [$documentId]
            );

            $body = json_encode($bodyCreate, JSON_PRETTY_PRINT);
            
            // Request body
            $request->setBody($body);
            
            $return['request']['parameters'] = $parameters;
            $return['request']['body'] = json_decode($body, true);
            
            try {
                $response = $request->send();

                $retorno =  $response->getBody();
                $retorno =  json_decode($retorno, true);
                
                $return['response'] = $retorno;
                
                return $return;
            }
            catch (\HTTP_Request2_Exception $ex) {
                $return['response'] = $ex;
                return $return;
            }
        }
        catch(\Exception $e){
            throw $e;
        }
    }
    
    public function excluirdocumento($documentId) {
        
        $token = $this->getTokenCertsign();
        
        $return = array();

        try {
            $request = new \Http_Request2($this->urlIntegracaoCertisign.'document/delete/'.$documentId, \Http_Request2::METHOD_DELETE);
            $url = $request->getUrl();
    
            $headers = array(
                // Request headers
                'Token' => $token,
            );
            
            $request->setHeader($headers);
            
            $parameters = array(
                // Request parameters
            );
            
            $url->setQueryVariables($parameters);
            
            $request->setMethod(\Http_Request2::METHOD_DELETE);
            $body = null;
            // Request body
            $request->setBody($body);
            
            $return['request']['parameters'] = $parameters;
            $return['request']['body'] = $body;
            
            try {
                $response = $request->send();
                
                $retorno = $response->getBody();
                $retorno = json_decode($retorno, true);
            
                $return['response'] = $retorno;
                return $return;
            }
            catch (\HTTP_Request2_Exception $ex)
            {
                $return['response'] = $ex;
    
                return $return;
            }
        } catch(\Exception $e){
            throw $e;
        }
    }
    
    public function dowloadDocumento($chave) {
        
        $token = $this->getTokenCertsign();
        
        $return = array();

        try {
            $request = new \Http_Request2($this->urlIntegracaoCertisign.'document/package?key='.$chave, \Http_Request2::METHOD_GET);
            $url = $request->getUrl();
    
            $headers = array(
                // Request headers
                'Token' => $token,
            );
            
            $request->setHeader($headers);
            
            $parameters = array(
                // Request parameters
                'key' => $chave,
                'includeOriginal' => 'True',
                'includeManifest' => 'True',
                'zipped' => 'True'
            );
            
            $url->setQueryVariables($parameters);
            
            $request->setMethod(\Http_Request2::METHOD_GET);
            $body = null;
            // Request body
            $request->setBody($body);
            
            $return['request']['parameters'] = $parameters;
            $return['request']['body'] = $body;
            
             try {
                $response = $request->send();
                
                $retorno =  $response->getBody();
                $retorno =  json_decode($retorno, true);
                
                $return['response'] = $retorno;
                
                return $return;
            }
            catch (\HTTP_Request2_Exception $ex) {
                $return['response'] = $ex;
                return $return;
            }
        }
        catch(\Exception $e){
            throw $e;
        }
    }
    
    public function verificarAssinaturaCda($chave) {
        
        $token = $this->getTokenCertsign();
        
        $return = array();

        try {
            $request = new \Http_Request2($this->urlIntegracaoCertisign.'document/ValidateSignatures?key='.$chave, \Http_Request2::METHOD_GET);
            $url = $request->getUrl();
    
            $headers = array(
                // Request headers
                'Token' => $token,
            );
            
            $request->setHeader($headers);
            
            $parameters = array(
                // Request parameters
                'key' => $chave 
            );
            
            $url->setQueryVariables($parameters);
            
            $request->setMethod(\Http_Request2::METHOD_GET);
            $body = null;
            // Request body
            $request->setBody($body);
            
            $return['request']['parameters'] = $parameters;
            $return['request']['body'] = $body;
            
             try {
                $response = $request->send();
                
                $retorno =  $response->getBody();
                $retorno =  json_decode($retorno, true);

                $return['response'] = $retorno;
                
                return $return['response'];
            }
            catch (\HTTP_Request2_Exception $ex) {
                $return['response'] = $ex;
                return $return;
            }
        }
        catch(\Exception $e){
            throw $e;
        }
    }

    public function getTokenCertsign() {
        
        try {
           
           $nrorg = $this->environment->getNrOrg();
               
           $params = array(
                "NRORG" => $nrorg
           );
            
           $token = $this->assinaturaFactory->getTokenCertsign($params);
        
           $token = $token[0]["DSTOKENCERTSIGN"];   
           
           return $token;    
        
        } catch(\Exception $e) {
            throw $e;
        }   
    }
    
}