<?php
namespace SM\Service;

use SM\Factory\ConfContratoMotivoRescisaoFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;

class ConfContratoMotivoRescisaoService
{

    protected $motivoRescisaoFactory;
    protected $environment;
    protected $newCode;

    public function __construct(EntityManager $entityManager, ConfContratoMotivoRescisaoFactory $motivoRescisaoFactory, Environment $environment, NewCode $newCode)
    {
        $this->connection            = $entityManager->getConnection();
        $this->motivoRescisaoFactory = $motivoRescisaoFactory;
        $this->environment           = $environment;
        $this->newCode               = $newCode;
    }

    public function getMotivoRecisao()
    {
        return $this->motivoRescisaoFactory->getMotivoRecisao();
    }

    public function salvarMotivos($params)
    {
        $this->connection->beginTransaction();
        try {
            $nrorg = $this->environment->getNrorg();
            $operador = $this->environment->getCdOperador();
            foreach ($params["DRG"] as $row) {
                if ($row["NRMOTIVORESCISAO"] != 0) {
                    $parametros = array (
                        "NRMOTIVORESCISAO"      => $row["NRMOTIVORESCISAO"],
                        "IDATIVO"               => $row["IDATIVO"],
                        "DSMOTIVORESCISAO"      => $row['DSMOTIVORESCISAO'],
                        "NRORGULTATU"           => $nrorg,
                        "CDOPERULTATU"          => $operador
                    );

                    $this->motivoRescisaoFactory->editarMotivos($parametros);
                } else {
                    $nrmotivorescisao = $this->newCode->getCodeByTableName("FCT_MOTIVORESCISAO", $nrorg, 1, 10, true);
                    
                    $parametros = array (
                        "NRMOTIVORESCISAO"      => $nrmotivorescisao,
                        "IDATIVO"               => $row['IDATIVO'],
                        "DSMOTIVORESCISAO"      => $row['DSMOTIVORESCISAO'],
                        "NRORG"                 => $nrorg,
                        "NRORGINCLUSAO"         => $nrorg,
                        "CDOPERINCLUSAO"        => $operador,
                    );
                    $this->motivoRescisaoFactory->salvarMotivos($parametros);
                }
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function excluirMotivos($parametro)
    {
        $param = implode(',', $parametro["NRS"]);
        $this->motivoRescisaoFactory->excluirMotivos($param);
    }
}
