<?php

namespace SM\Service;

use SM\Factory\EscopoFaturamentoFactory;
use SM\Service\FindService;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;

class EscopoFaturamentoService
{
    protected $EscopoFaturamentoFactory;
    protected $findService;
    protected $Environment;
    protected $newCode;
    protected $connection;

    public function __construct(EntityManager $entityManager, EscopoFaturamentoFactory $EscopoFaturamentoFactory, FindService $findService, Environment $Environment, NewCode $newCode)
    {
        $this->connection                   = $entityManager->getConnection();
        $this->EscopoFaturamentoFactory     = $EscopoFaturamentoFactory;
        $this->findService                  = $findService;
        $this->Environment                  = $Environment;
        $this->newCode                      = $newCode;
    }

    public function editEscopoFaturamento($params)
    {
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->Environment->getNrOrg();
            $cdoperador = $this->Environment->getCdOperador();

            $dataParams = array(
                "NRTIPOORCAMENTO"       => $params["NRTIPOORCAMENTO"],
                "NMTIPOORCAMENTO"       => $params["NMTIPOORCAMENTO"], 
                "IDTIPOORCA"            => $params["IDTIPOORCA"],
                "IDALTQTDMEDIDAPROP"    => $params["IDALTQTDMEDIDAPROP"],
                "IDALTVRTABRPECPROP"    => $params["IDALTVRTABRPECPROP"],
                "IDUTILIZAVARPRECO"     => $params["IDUTILIZAVARPRECO"],
                "IDFILTRASERVICO"       => $params["IDFILTRASERVICO"],
                "IDATIVO"               => $params["IDATIVO"],
                "NRORG"                 => $nrorg,
                "CDOPERULTATU"          => $cdoperador,
            );

            $this->EscopoFaturamentoFactory->editEscopoFaturamento($dataParams);
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    } 

    public function saveAddEscopoFaturamento($params)
    {
        $nrorg = $this->Environment->getNrOrg();
        $cdoperador = $this->Environment->getCdOperador();

        $this->connection->beginTransaction();

        try {
            
            $nrtipoorcamento = $this->newCode->getCodeByTableName("FCT_TIPOORCAMENTO", $nrorg, 1, 10, true);
            
            $dataParams = array (
                "NRTIPOORCAMENTO"       => $nrtipoorcamento,
                "NMTIPOORCAMENTO"       => $params["NMTIPOORCAMENTO"],
                "IDGERACONTRATO"        => 'S',
                "IDATIVO"               => $params["IDATIVO"],
                "IDTIPOORCA"            => $params["IDTIPOORCA"],
                "IDALTQTDMEDIDAPROP"    => $params["IDALTQTDMEDIDAPROP"],
                "IDALTVRTABRPECPROP"    => $params["IDALTVRTABRPECPROP"],
                "IDUTILIZAVARPRECO"     => $params["IDUTILIZAVARPRECO"],
                "IDFILTRASERVICO"       => $params["IDFILTRASERVICO"],
                "NRORG"                 => $nrorg,
                "CDOPERINCLUSAO"        => $cdoperador,
            );
        
            $this->EscopoFaturamentoFactory->saveAddEscopoFaturamento($dataParams);
        
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function deleteEscopoFaturamento($parametros)
    {
        try {

            foreach ($parametros as $parametroEscopo) {

                $this->connection->beginTransaction();

                $param = array(
                    "NRTIPOORCAMENTO"   => $parametroEscopo['NRTIPOORCAMENTO']
                );

                $dadosTabelaPrecoEscopoFat = $this->EscopoFaturamentoFactory->getTabelaPrecoEscopoFat($param);
                $dadosItEscopoProposta = $this->EscopoFaturamentoFactory->getItEscopoProposta($param);
                $dadosCapaParamFatEsc = $this->EscopoFaturamentoFactory->getCapaParamFatEsc($param);
                $dadosIndicesPadrEscopo = $this->EscopoFaturamentoFactory->getIndicesPadrEscopo($param);

                if (isset($dadosTabelaPrecoEscopoFat["data"][0]) || isset($dadosItEscopoProposta["data"][0]) || isset($dadosCapaParamFatEsc["data"][0]) || isset($dadosIndicesPadrEscopo["data"][0])) {
                    throw new \Exception("EXISTE");
                }

                $this->EscopoFaturamentoFactory->deleteEscopoFaturamento($param);
                $this->connection->commit();
            }
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function getTabelaPrecoEscopoFat($params)
    {
        $params = array(
            "NRTIPOORCAMENTO"   => $params['NRTIPOORCAMENTO']
        );
        return $this->EscopoFaturamentoFactory->getTabelaPrecoEscopoFat($params);
    }
    
    public function getMoedaTabelaPrecoEscopoFat($params)
    {
        return $this->EscopoFaturamentoFactory->getMoedaTabelaPrecoEscopoFat($params);
    } 

    public function getPrecBaseServTabelaPrecoEscopoFat($params)
    {
        return $this->EscopoFaturamentoFactory->getPrecBaseServTabelaPrecoEscopoFat($params);
    }

    public function getPrecBaseTabelaPrecoEscopoFat($params)
    {
        return $this->EscopoFaturamentoFactory->getPrecBaseTabelaPrecoEscopoFat($params);
    }

    public function saveTabelaPreco($params)
    {
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->Environment->getNrOrg();
            $cdoperador = $this->Environment->getCdOperador();
            $novoCodigo = $this->newCode->getCodeByTableName("FCT_TABPRECESCOPOFAT", $this->Environment->getNrOrg(), 1, 10, true);

            $dataParamsAux = array(
                "NRTIPOORCAMENTO"       => $params["NRTIPOORCAMENTO"],
            );

            $dados = $this->EscopoFaturamentoFactory->getValidaMoeda($dataParamsAux);
            //Nova Tabela de Preço
            if (!isset($params["NRTABPRECESCOPOFAT"])) {

                foreach ($dados["data"] as $moeda) {
                    if ($moeda["NRMOEDA"] == $params["NRMOEDA"]) {
                        throw new \Exception("EXISTE");
                    }
                }

                $dataParams = array(
                    "NRTABPRECESCOPOFAT"    => $novoCodigo,
                    "NRMOEDA"               => $params["NRMOEDA"],
                    "NRTIPOORCAMENTO"       => $params["NRTIPOORCAMENTO"],
                    "CDPRECBASESERV"        => isset($params["CDPRECBASESERV"])? $params["CDPRECBASESERV"] : null,
                    "CDPRECBASEVEND"        => isset($params["CDPRECBASEVEND"])? $params["CDPRECBASEVEND"] : null,
                    "IDATIVO"               => 'S',
                    "NRORG"                 => $nrorg,
                    "CDOPERINCLUSAO"          => $cdoperador,
                );

                $this->EscopoFaturamentoFactory->addTabelaPreco($dataParams);
                $this->connection->commit();

            } else {
                // Edição de um item na tabela de preço
                foreach ($dados["data"] as $moeda) {
                    if (($moeda["NRMOEDA"] == $params["NRMOEDA"]) && $params["NRTABPRECESCOPOFAT"] == $moeda["NRTABPRECESCOPOFAT"]) {
                        $dataParams = array(
                            "NRTABPRECESCOPOFAT"    => $params["NRTABPRECESCOPOFAT"],
                            "NRMOEDA"               => $params["NRMOEDA"],
                            "CDPRECBASESERV"        => isset($params["CDPRECBASESERV"])? $params["CDPRECBASESERV"] : null,
                            "CDPRECBASEVEND"        => isset($params["CDPRECBASEVEND"])? $params["CDPRECBASEVEND"] : null,
                            "NRORG"                 => $nrorg,
                            "CDOPERULTATU"          => $cdoperador,
                        );
                    } else if (($moeda["NRMOEDA"] != $params["NRMOEDA"]) && $params["NRTABPRECESCOPOFAT"] == $moeda["NRTABPRECESCOPOFAT"]) {
                        foreach ($dados["data"] as $moeda) {
                            if ($moeda["NRMOEDA"] == $params["NRMOEDA"]) {
                                throw new \Exception("EXISTE");
                            }
                        }
                        
                        $dataParams = array(
                            "NRTABPRECESCOPOFAT"    => $params["NRTABPRECESCOPOFAT"],
                            "NRMOEDA"               => $params["NRMOEDA"],
                            "CDPRECBASESERV"        => isset($params["CDPRECBASESERV"])? $params["CDPRECBASESERV"] : null,
                            "CDPRECBASEVEND"        => isset($params["CDPRECBASEVEND"])? $params["CDPRECBASEVEND"] : null,
                            "NRORG"                 => $nrorg,
                            "CDOPERULTATU"          => $cdoperador,
                        );
                    }
                }

                $this->EscopoFaturamentoFactory->editTabelaPreco($dataParams);
                $this->connection->commit();
            }   
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function deleteTabPrecEscopoFat($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $retorno = $this->findService->formatarArray($parametros["NRS"]);
            $this->EscopoFaturamentoFactory->deleteTabPrecEscopoFat($retorno);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
    
    public function getItEscopoProposta($params)
    {
        $params = array(
            "NRTIPOORCAMENTO"   => $params['NRTIPOORCAMENTO']
        );
        return $this->EscopoFaturamentoFactory->getItEscopoProposta($params);
    }

    public function getParamFatItEsc($params)
    {
        $dataParams = array(
            "NRITESCOPOPROPOSTA"   => $params['NRITESCOPOPROPOSTA']
        );
        return $this->EscopoFaturamentoFactory->getParamFatItEsc($dataParams);
    }

    public function getEspFatServicoOrca()
    {
        return $this->EscopoFaturamentoFactory->getEspFatServicoOrca();
    }

    public function saveEscFatServicoEscopo($params)
    {
        try {

            $nrorg = $this->Environment->getNrOrg();
            $cdoperador = $this->Environment->getCdOperador();
            
            if ($params['NRITESCOPOPROPOSTA'] == null) {
                
                $this->connection->beginTransaction();

                foreach ($params["CDSERVORCA"] as $cdservorca) {
    
                    $paramsValida = array(
                        "CDSERVORCA"        => $cdservorca,
                        "NRTIPOORCAMENTO"   => $params['NRTIPOORCAMENTO'],
                        "NRORG"             => $nrorg,
                    );
            
                    $valor = $this->EscopoFaturamentoFactory->validaServicoEscopoDuplicados($paramsValida);
            
                    if ($valor[0]["VALIDASERV"] > 0) {
                        throw new \Exception("EXISTE");
                    }
    
                    $novoCodigo = $this->newCode->getCodeByTableName("FCT_ITESCOPOPROPOSTA", $this->Environment->getNrOrg(), 1, 10, true);
                    
                    $dataParams = array(
                        "NRITESCOPOPROPOSTA"    => $novoCodigo,
                        "NRTIPOORCAMENTO"       => $params['NRTIPOORCAMENTO'],
                        "CDSERVORCA"            => $cdservorca,
                        "NRORG"                 => $nrorg,
                        "IDATIVO"               => 'S',
                        "CDOPERINCLUSAO"        => $cdoperador,
                        "IDPREVCOBRFUT"         => $params['IDPREVCOBRFUT']
                    );
                    $this->EscopoFaturamentoFactory->insertEscFatServicoEscopo($dataParams);
                }
            } else {
                $this->connection->beginTransaction();

                $dataParams = array(
                    "NRITESCOPOPROPOSTA"    => $params['NRITESCOPOPROPOSTA'],
                    "CDSERVORCA"            => $params['CDSERVORCA'],
                    "NRORGULTATU"           => $nrorg,
                    "CDOPERULTATU"          => $cdoperador,
                    "IDPREVCOBRFUT"         => $params['IDPREVCOBRFUT']
                );

                $this->EscopoFaturamentoFactory->editEscFatServicoEscopo($dataParams);
            }
            
            $this->connection->commit();  

        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function deleteEscFatItServicoEscopo($parametros)
    {
        $this->connection->beginTransaction();

        try {

            foreach ($parametros as $parametroItServico) {
                $dataParam = array( "NRITESCOPOPROPOSTA" => $parametroItServico["NRITESCOPOPROPOSTA"],);
                $this->EscopoFaturamentoFactory->deleteEscFatItServicoEscopo($dataParam);
            }
            $this->connection->commit();

        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function getCapaParamFatEsc($params)
    {
        $params = array(
            "NRTIPOORCAMENTO"   => $params['NRTIPOORCAMENTO']
        );
        return $this->EscopoFaturamentoFactory->getCapaParamFatEsc($params);
    } 

    public function saveCapaParametrosFaturamento($params) 
    {
        $nrorg = $this->Environment->getNrOrg();
        $cdoperador = $this->Environment->getCdOperador();

        $this->connection->beginTransaction();

        try {
            if ($params['NRCAPAPARAMFATESC'] == null) {

                $paramsValida = array(
                    "NRTIPOORCAMENTO"       => $params["NRTIPOORCAMENTO"],
                );
        
                $valor = $this->EscopoFaturamentoFactory->validaCapaParamFatDuplicados($paramsValida);
        
                if ($valor[0]["QTD"] > 0) {
                    throw new \Exception("EXISTE");
                }

                $nrcapaparamfatesc = $this->newCode->getCodeByTableName("FCT_CAPAPARAMFATESC", $nrorg, 1, 10, true);
                $dataParams = array (
                    "NRCAPAPARAMFATESC"         => $nrcapaparamfatesc,
                    "NRTIPOORCAMENTO"           => $params["NRTIPOORCAMENTO"],
                    "CDPRODFATURA"              => $params["CDPRODFATURA"],
                    "CDFILIALFATURA"            => $params["CDFILIALFATURA"],
                    "NRPARCFATURA"              => $params["NRPARCFATURA"],
                    "IDTPFATURA"                => $params["IDTPFATURA"],
                    "QTPARCELASFATU"            => $params["QTPARCELASFATU"],
                    "IDTPCALCPARCINC"           => $params["IDTPCALCPARCINC"],
                    "IDGERARATEIO"              => 'N',
                    "IDAPURAAPONTAMENTO"        => 'N',
                    "IDAGRUPPARC"               => $params["IDAGRUPPARC"],
                    "QTDIASAPURACAO"            => $params["QTDIASAPURACAO"],
                    "NRDIACALCVENC"             => $params["NRDIACALCVENC"],
                    "QTDIASFATUVENC"            => $params["QTDIASFATUVENC"],
                    "IDTPCALCVENC"              => $params["IDTPCALCVENC"],
                    "IDMESCALCVENC"             => $params["IDMESCALCVENC"],
                    "CDTIPOOPERFATURA"          => $params["CDTIPOOPERFATURA"],
                    "CDEMPRESA"                 => $params["CDEMPRESA"],
                    "CDSERIENF"                 => $params["CDSERIENF"],
                    "CDINSCESTA"                => $params["CDINSCESTA"],
                    "CDINSCMUNI"                => $params["CDINSCMUNI"],
                    "IDTPCALCDTSAIDANF"         => $params["IDTPCALCDTSAIDANF"],
                    "NRCALCULODTSAIDANF"        => $params["NRCALCULODTSAIDANF"],
                    "IDMUNIPRESTSERV"           => $params["IDMUNIPRESTSERV"],
                    "IDTIPONATUOPER"            => $params["IDTIPONATUOPER"],
                    "IDNATUOPER"                => $params["IDNATUOPER"],
                    "CDMODDOCFISCFATURA"        => $params["CDMODDOCFISCFATURA"],
                    "CDTPMEDIDAVARIAVEL"        => $params["CDTPMEDIDAVARIAVEL"],
                    "DSSERVORCABOL"             => $params["DSSERVORCABOL"],
                    "DSSERVORCANF"              => $params["DSSERVORCANF"],
                    "QTPARCELASPARAUTO"         => $params["QTPARCELASPARAUTO"],
                    "QTDIASINTERVALOPARAUTO"    => $params["QTDIASINTERVALOPARAUTO"],
                    "NRDIAVENCIMENTOPARAUTO"    => $params["NRDIAVENCIMENTOPARAUTO"],
                    "QTCARENCIADIASPARAUTO"     => $params["QTCARENCIADIASPARAUTO"],
                    "IDATIVO"                   => 'S',
                    "NRORG"                     => $nrorg,
                    "CDOPERINCLUSAO"            => $cdoperador,
                );
            
                $this->EscopoFaturamentoFactory->insertCapaParametrosFaturamento($dataParams);
            } else {
                $dataParams = array (
                    "NRCAPAPARAMFATESC"         => $params["NRCAPAPARAMFATESC"],
                    "NRTIPOORCAMENTO"           => $params["NRTIPOORCAMENTO"],
                    "CDPRODFATURA"              => $params["CDPRODFATURA"],
                    "CDFILIALFATURA"            => $params["CDFILIALFATURA"],
                    "NRPARCFATURA"              => $params["NRPARCFATURA"],
                    "IDTPFATURA"                => $params["IDTPFATURA"],
                    "QTPARCELASFATU"            => $params["QTPARCELASFATU"],
                    "IDTPCALCPARCINC"           => $params["IDTPCALCPARCINC"],
                    "IDGERARATEIO"              => 'N',
                    "IDAPURAAPONTAMENTO"        => 'N',
                    "IDAGRUPPARC"               => $params["IDAGRUPPARC"],
                    "QTDIASAPURACAO"            => $params["QTDIASAPURACAO"],
                    "NRDIACALCVENC"             => $params["NRDIACALCVENC"],
                    "QTDIASFATUVENC"            => $params["QTDIASFATUVENC"],
                    "IDTPCALCVENC"              => $params["IDTPCALCVENC"],
                    "IDMESCALCVENC"             => $params["IDMESCALCVENC"],
                    "CDTIPOOPERFATURA"          => $params["CDTIPOOPERFATURA"],
                    "CDEMPRESA"                 => $params["CDEMPRESA"],
                    "CDSERIENF"                 => $params["CDSERIENF"],
                    "CDINSCESTA"                => $params["CDINSCESTA"],
                    "CDINSCMUNI"                => $params["CDINSCMUNI"],
                    "IDTPCALCDTSAIDANF"         => $params["IDTPCALCDTSAIDANF"],
                    "NRCALCULODTSAIDANF"        => $params["NRCALCULODTSAIDANF"],
                    "IDMUNIPRESTSERV"           => $params["IDMUNIPRESTSERV"],
                    "IDTIPONATUOPER"            => $params["IDTIPONATUOPER"],
                    "IDNATUOPER"                => $params["IDNATUOPER"],
                    "CDMODDOCFISCFATURA"        => $params["CDMODDOCFISCFATURA"],
                    "CDTPMEDIDAVARIAVEL"        => $params["CDTPMEDIDAVARIAVEL"],
                    "DSSERVORCABOL"             => $params["DSSERVORCABOL"],
                    "DSSERVORCANF"              => $params["DSSERVORCANF"],
                    "QTPARCELASPARAUTO"         => $params["QTPARCELASPARAUTO"],
                    "QTDIASINTERVALOPARAUTO"    => $params["QTDIASINTERVALOPARAUTO"],
                    "NRDIAVENCIMENTOPARAUTO"    => $params["NRDIAVENCIMENTOPARAUTO"],
                    "QTCARENCIADIASPARAUTO"     => $params["QTCARENCIADIASPARAUTO"],
                    "IDATIVO"                   => 'S',
                    "NRORG"                     => $nrorg,
                    "CDOPERULTATU"              => $cdoperador,
                );

                $this->EscopoFaturamentoFactory->updateCapaParametrosFaturamento($dataParams);
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function deleteCapaParametrosFaturamento($parametros)
    {
        $this->connection->beginTransaction();

        try {
            
            $dataParam = array( 
                "NRTIPOORCAMENTO" => $parametros["NRTIPOORCAMENTO"],
            );
            $this->EscopoFaturamentoFactory->deleteCapaParametrosFaturamento($dataParam);
            
            $this->connection->commit();

        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function saveParamFatCompetencia($params) 
    {
        $nrorg = $this->Environment->getNrOrg();
        $cdoperador = $this->Environment->getCdOperador();

        $this->connection->beginTransaction();

        try {
            if ($params['NRPARAMFATESC'] == null) {

                $paramsValida = array(
                    "NRTIPOORCAMENTO"   => $params['NRTIPOORCAMENTO'],
                );
        
                $valor = $this->EscopoFaturamentoFactory->validaParamFatCompetenciaDuplicados($paramsValida);
        
                if ($valor[0]["QTD"] > 0) {
                    throw new \Exception("EXISTE");
                }

                $nrparamfatesc = $this->newCode->getCodeByTableName("FCT_PARAMFATESC", $nrorg, 1, 10, true);
                $dataParams = array (
                    "NRPARAMFATESC"         => $nrparamfatesc,
                    "NRTIPOORCAMENTO"       => $params["NRTIPOORCAMENTO"], 
                    "NRDIACOMPETINI"        => $params["NRDIACOMPETINI"],
                    "IDMESCOMPETINI"        => $params["IDMESCOMPETINI"],
                    "IDMESCOMPETFIN"        => $params["IDMESCOMPETFIN"],
                    "NRDIACOMPETFIN"        => $params["NRDIACOMPETFIN"],
                    "QTDIASAPURACAO"        => $params["QTDIASAPURACAO"],
                    "IDTPCALCVENC"          => $params["IDTPCALCVENC"],
                    "NRDIACALCVENC"         => $params["NRDIACALCVENC"],
                    "IDMESCALCVENC"         => $params["IDMESCALCVENC"],
                    "QTDIASFATUVENC"        => $params["QTDIASFATUVENC"],
                    "IDATIVO"               => 'S',
                    "NRORG"                 => $nrorg,
                    "CDOPERINCLUSAO"        => $cdoperador,
                );
            
                $this->EscopoFaturamentoFactory->insertParamFatCompetencia($dataParams);
            } else {
                $dataParams = array (
                    "NRPARAMFATESC"         => $params["NRPARAMFATESC"], 
                    "NRTIPOORCAMENTO"       => $params["NRTIPOORCAMENTO"], 
                    "NRDIACOMPETINI"        => $params["NRDIACOMPETINI"],
                    "IDMESCOMPETINI"        => $params["IDMESCOMPETINI"],
                    "IDMESCOMPETFIN"        => $params["IDMESCOMPETFIN"],
                    "NRDIACOMPETFIN"        => $params["NRDIACOMPETFIN"],
                    "QTDIASAPURACAO"        => $params["QTDIASAPURACAO"],
                    "IDTPCALCVENC"          => $params["IDTPCALCVENC"],
                    "NRDIACALCVENC"         => $params["NRDIACALCVENC"],
                    "IDMESCALCVENC"         => $params["IDMESCALCVENC"],
                    "QTDIASFATUVENC"        => $params["QTDIASFATUVENC"],
                    "IDATIVO"               => 'S',
                    "NRORG"                 => $nrorg,
                    "CDOPERULTATU"          => $cdoperador,
                );
                
                $this->EscopoFaturamentoFactory->updateParamFatCompetencia($dataParams);
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function deleteParamFatCompetencia($parametros)
    {
        $this->connection->beginTransaction();

        try {

            $dataparams = array(
                "NRPARAMFATESC" => $parametros[0]["NRPARAMFATESC"],
            );
            $this->EscopoFaturamentoFactory->deleteParamFatCompetencia($dataparams);
            
            $this->connection->commit();

        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function getTipoMedidaVariavelParamFat()
    {
        return $this->EscopoFaturamentoFactory->getTipoMedidaVariavelParamFat();
    }

    public function getProdutoParaFaturamentoParamFat()
    {
        return $this->EscopoFaturamentoFactory->getProdutoParaFaturamentoParamFat();
    } 

    public function getFilialParamFat()
    {
        return $this->EscopoFaturamentoFactory->getFilialParamFat();
    } 

    public function getTipoOperacParamFat()
    {
        return $this->EscopoFaturamentoFactory->getTipoOperacParamFat();
    } 

    public function getSerieNotaParamFat($params)
    {
        $params = array(
            "CDEMPRESA" => isset($params['CDEMPRESA'])? $params['CDEMPRESA']: null,
        );
        return $this->EscopoFaturamentoFactory->getSerieNotaParamFat($params);
    } 

    public function getParamfatitescserv($params)
    {
        $params = array(
            "NRITESCOPOPROPOSTA" => $params['NRITESCOPOPROPOSTA'],
        );
        return $this->EscopoFaturamentoFactory->getParamfatitescserv($params);
    }

    public function getParamfatesc($params)
    {
        $params = array(
            "NRTIPOORCAMENTO" => $params['NRTIPOORCAMENTO'],
        );
        return $this->EscopoFaturamentoFactory->getParamfatesc($params);
    }

    public function saveServEscTabParamFat($params) 
    {
        $nrorg = $this->Environment->getNrOrg();
        $cdoperador = $this->Environment->getCdOperador();

        $this->connection->beginTransaction();

        try {
            if ($params['NRPARAMFATITESC'] == null) {

                $paramsValida = array(
                    "NRITESCOPOPROPOSTA"   => $params['NRITESCOPOPROPOSTA'],
                    "NRTIPOORCAMENTO"       => $params["NRTIPOORCAMENTO"],
                );
        
                $valor = $this->EscopoFaturamentoFactory->validaServEscParamFatDuplicados($paramsValida);
        
                if ($valor[0]["QTD"] > 0) {
                    throw new \Exception("EXISTE");
                }
 
                $nrparamfatitesc = $this->newCode->getCodeByTableName("FCT_PARAMFATITESC", $nrorg, 1, 10, true);
                $dataParams = array (
                    "NRPARAMFATITESC"           => $nrparamfatitesc,
                    "NRITESCOPOPROPOSTA"        => $params["NRITESCOPOPROPOSTA"], 
                    "NRTIPOORCAMENTO"           => $params["NRTIPOORCAMENTO"],
                    "CDPRODFATURA"              => $params["CDPRODFATURA"],
                    "CDFILIALFATURA"            => $params["CDFILIALFATURA"],
                    "NRPARCFATURA"              => $params["NRPARCFATURA"],
                    "IDTPFATURA"                => $params["IDTPFATURA"],
                    "QTPARCELASFATU"            => $params["QTPARCELASFATU"],
                    "QTDIASAPURACAO"            => $params["QTDIASAPURACAO"],
                    "NRDIACALCVENC"             => $params["NRDIACALCVENC"],
                    "QTDIASFATUVENC"            => $params["QTDIASFATUVENC"],
                    "IDTPCALCVENC"              => $params["IDTPCALCVENC"],
                    "IDMESCALCVENC"             => $params["IDMESCALCVENC"],
                    "CDTIPOOPERFATURA"          => $params["CDTIPOOPERFATURA"],
                    "CDSERIENF"                 => $params["CDSERIENF"],
                    "CDMODDOCFISCFATURA"        => $params["CDMODDOCFISCFATURA"],
                    "CDEMPRESA"                 => $params["CDEMPRESA"],
                    "CDINSCESTA"                => $params["CDINSCESTA"],
                    "CDINSCMUNI"                => $params["CDINSCMUNI"],
                    "CDTPMEDIDAVARIAVEL"        => $params["CDTPMEDIDAVARIAVEL"],
                    "DSSERVORCABOL"             => $params["DSSERVORCABOL"],
                    "DSSERVORCANF"              => $params["DSSERVORCANF"],
                    "QTPARCELASPARAUTO"         => $params["QTPARCELASPARAUTO"],
                    "QTDIASINTERVALOPARAUTO"    => $params["QTDIASINTERVALOPARAUTO"],
                    "NRDIAVENCIMENTOPARAUTO"    => $params["NRDIAVENCIMENTOPARAUTO"],
                    "QTCARENCIADIASPARAUTO"     => $params["QTCARENCIADIASPARAUTO"],
                    "IDATIVO"                   => 'S',
                    "NRORG"                     => $nrorg,
                    "CDOPERINCLUSAO"            => $cdoperador,
                );
            
                $this->EscopoFaturamentoFactory->insertServEscTabParamFat($dataParams);
            } else {
                $dataParams = array (
                    "NRPARAMFATITESC"           => $params["NRPARAMFATITESC"],
                    "NRITESCOPOPROPOSTA"        => $params["NRITESCOPOPROPOSTA"], 
                    "NRTIPOORCAMENTO"           => $params["NRTIPOORCAMENTO"],
                    "CDPRODFATURA"              => $params["CDPRODFATURA"],
                    "CDFILIALFATURA"            => $params["CDFILIALFATURA"],
                    "NRPARCFATURA"              => $params["NRPARCFATURA"],
                    "IDTPFATURA"                => $params["IDTPFATURA"],
                    "QTPARCELASFATU"            => $params["QTPARCELASFATU"],
                    "QTDIASAPURACAO"            => $params["QTDIASAPURACAO"],
                    "NRDIACALCVENC"             => $params["NRDIACALCVENC"],
                    "QTDIASFATUVENC"            => $params["QTDIASFATUVENC"],
                    "IDTPCALCVENC"              => $params["IDTPCALCVENC"],
                    "IDMESCALCVENC"             => $params["IDMESCALCVENC"],
                    "CDTIPOOPERFATURA"          => $params["CDTIPOOPERFATURA"],
                    "CDSERIENF"                 => $params["CDSERIENF"],
                    "CDMODDOCFISCFATURA"        => $params["CDMODDOCFISCFATURA"],
                    "CDEMPRESA"                 => $params["CDEMPRESA"],
                    "CDINSCESTA"                => $params["CDINSCESTA"],
                    "CDINSCMUNI"                => $params["CDINSCMUNI"],
                    "CDTPMEDIDAVARIAVEL"        => $params["CDTPMEDIDAVARIAVEL"],
                    "DSSERVORCABOL"             => $params["DSSERVORCABOL"],
                    "DSSERVORCANF"              => $params["DSSERVORCANF"],
                    "QTPARCELASPARAUTO"         => $params["QTPARCELASPARAUTO"],
                    "QTDIASINTERVALOPARAUTO"    => $params["QTDIASINTERVALOPARAUTO"],
                    "NRDIAVENCIMENTOPARAUTO"    => $params["NRDIAVENCIMENTOPARAUTO"],
                    "QTCARENCIADIASPARAUTO"     => $params["QTCARENCIADIASPARAUTO"],
                    "IDATIVO"                   => 'S',
                    "NRORG"                     => $nrorg,
                    "CDOPERULTATU"              => $cdoperador,
                );

                $this->EscopoFaturamentoFactory->updateServEscTabParamFat($dataParams);
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
    
    public function deleteServEscTabParamFat($parametros)
    {
        $this->connection->beginTransaction();

        try {
            
            $dataParam = array( 
                "NRITESCOPOPROPOSTA" => $parametros["NRITESCOPOPROPOSTA"],
            );
            $this->EscopoFaturamentoFactory->deleteServEscTabParamFat($dataParam);
            
            $this->connection->commit();

        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function saveServEscTabCompetencia($params) 
    {
        $nrorg = $this->Environment->getNrOrg();
        $cdoperador = $this->Environment->getCdOperador();

        $this->connection->beginTransaction();

        try {
            if ($params['NRPARAMFATITESCSERV'] == null) {

                $paramsValida = array(
                    "NRITESCOPOPROPOSTA"   => $params['NRITESCOPOPROPOSTA'],
                );
        
                $valor = $this->EscopoFaturamentoFactory->validaServEscCompetenciaDuplicados($paramsValida);
        
                if ($valor[0]["QTD"] > 0) {
                    throw new \Exception("EXISTE");
                }
 
                $nrparamfatitescserv = $this->newCode->getCodeByTableName("FCT_PARAMFATITESCSERV", $nrorg, 1, 10, true);
                $dataParams = array (
                    "NRPARAMFATITESCSERV"   => $nrparamfatitescserv,
                    "NRITESCOPOPROPOSTA"    => $params["NRITESCOPOPROPOSTA"], 
                    "NRDIACOMPETINI"        => $params["NRDIACOMPETINI"],
                    "IDMESCOMPETINI"        => $params["IDMESCOMPETINI"],
                    "IDMESCOMPETFIN"        => $params["IDMESCOMPETFIN"],
                    "NRDIACOMPETFIN"        => $params["NRDIACOMPETFIN"],
                    "QTDIASAPURACAO"        => $params["QTDIASAPURACAO"],
                    "IDTPCALCVENC"          => $params["IDTPCALCVENC"],
                    "NRDIACALCVENC"         => $params["NRDIACALCVENC"],
                    "IDMESCALCVENC"         => $params["IDMESCALCVENC"],
                    "QTDIASFATUVENC"        => $params["QTDIASFATUVENC"],
                    "IDATIVO"               => 'S',
                    "NRORG"                 => $nrorg,
                    "CDOPERINCLUSAO"        => $cdoperador,
                );
            
                $this->EscopoFaturamentoFactory->insertServEscTabCompetencia($dataParams);
            } else {
                $dataParams = array (
                    "NRPARAMFATITESCSERV"   => $params["NRPARAMFATITESCSERV"], 
                    "NRITESCOPOPROPOSTA"    => $params["NRITESCOPOPROPOSTA"], 
                    "NRDIACOMPETINI"        => $params["NRDIACOMPETINI"],
                    "IDMESCOMPETINI"        => $params["IDMESCOMPETINI"],
                    "IDMESCOMPETFIN"        => $params["IDMESCOMPETFIN"],
                    "NRDIACOMPETFIN"        => $params["NRDIACOMPETFIN"],
                    "QTDIASAPURACAO"        => $params["QTDIASAPURACAO"],
                    "IDTPCALCVENC"          => $params["IDTPCALCVENC"],
                    "NRDIACALCVENC"         => $params["NRDIACALCVENC"],
                    "IDMESCALCVENC"         => $params["IDMESCALCVENC"],
                    "QTDIASFATUVENC"        => $params["QTDIASFATUVENC"],
                    "IDATIVO"               => 'S',
                    "NRORG"                 => $nrorg,
                    "CDOPERULTATU"          => $cdoperador,
                );
                
                $this->EscopoFaturamentoFactory->updateServEscTabCompetencia($dataParams);
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    } 

    public function deleteServEscTabCompetencia($parametros)
    {
        $this->connection->beginTransaction();

        try {

            foreach ($parametros as $parametroCompetencia) {
                $dataParam = array( "NRPARAMFATITESCSERV" => $parametroCompetencia["NRPARAMFATITESCSERV"],);
                $this->EscopoFaturamentoFactory->deleteServEscTabCompetencia($dataParam);
            }
            $this->connection->commit();

        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function getDescAgrupEscopo($params)
    {
        $params = array(
            "NRTIPOORCAMENTO" => $params['NRTIPOORCAMENTO'],
        );
        return $this->EscopoFaturamentoFactory->getDescAgrupEscopo($params);
    } 

    public function getIndicesPadrEscopo($params)
    {
        $params = array(
            "NRTIPOORCAMENTO" => $params['NRTIPOORCAMENTO'],
        );
        return $this->EscopoFaturamentoFactory->getIndicesPadrEscopo($params);
    }

    public function saveIndicesFinanceirosPadrEscopo($parametros)
    {
        $paramsValida = array(
            "NRINDICESPADRESCOPO"   => $parametros['NRINDICESPADRESCOPO'],
            "CDINDICE"              => $parametros['CDINDICE'],
            "NRTIPOORCAMENTO"       => $parametros["NRTIPOORCAMENTO"],
        );

        $valor = $this->EscopoFaturamentoFactory->validaRegistrosDuplicados($paramsValida);

        if ($valor[0]["QTD"] > 0) {
            throw new \Exception("EXISTE");
        }

        $this->connection->beginTransaction();
        $nrorg = $this->Environment->getNrOrg();
        $cdoperador = $this->Environment->getCdOperador();

        try {
            if ($parametros['NRINDICESPADRESCOPO'] == null) {
                $nrindicespadrescopo = $this->newCode->getCodeByTableName("FCT_INDICESPADRESCOPO", $nrorg, 1, 10, true);
                $dataParams = array (
                    "NRINDICESPADRESCOPO"   => $nrindicespadrescopo,
                    "CDINDICE"              => $parametros["CDINDICE"],
                    "CDTPINDICE"            => $parametros["CDTPINDICE"],
                    "IDPERVALOR"            => $parametros["IDPERVALOR"],
                    "NRPERCENTINDICE"       => $parametros["NRPERCENTINDICE"],
                    "NRTIPOORCAMENTO"       => $parametros["NRTIPOORCAMENTO"],
                    "NRORG"                 => $nrorg,
                    "CDOPERINCLUSAO"        => $cdoperador,
                    "IDATIVO"               => "S",
                );
                $this->EscopoFaturamentoFactory->insertIndiceFinanceiroPadrEscopo($dataParams);
            } else {
                $dataParams = array (
                    "NRINDICESPADRESCOPO"   => $parametros["NRINDICESPADRESCOPO"],
                    "CDINDICE"              => $parametros["CDINDICE"],
                    "CDTPINDICE"            => $parametros["CDTPINDICE"],
                    "IDPERVALOR"            => $parametros["IDPERVALOR"],
                    "NRPERCENTINDICE"       => $parametros["NRPERCENTINDICE"],
                    "NRTIPOORCAMENTO"       => $parametros["NRTIPOORCAMENTO"],
                    "NRORG"                 => $nrorg,
                    "CDOPERULTATU"          => $cdoperador,
                    "IDATIVO"               => "S",
                );
                $this->EscopoFaturamentoFactory->updateIndiceFinanceiroPadrEscopo($dataParams);
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function deleteParametrosIndicePadrEscopo($parametros)
    {
        $this->connection->beginTransaction();
        try {
            foreach ($parametros as $parametroIndices) {
                $dataParam = array( "NRINDICESPADRESCOPO" => $parametroIndices["NRINDICESPADRESCOPO"],);
                $this->EscopoFaturamentoFactory->deleteParametrosIndicePadrEscopo($dataParam);
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function saveEscFatDescricaoEscopo($params)
    {
        $paramsValida = array(
            "NRDESCAGRUPESCOPO" => $params['NRDESCAGRUPESCOPO'],
            "NRTIPOORCAMENTO"   => $params['NRTIPOORCAMENTO'],
            "IDTPFATURA"        => $params['IDTPFATURA'],
            "CDFILIALFATURA"    => $params['CDFILIALFATURA'],
            "NRPARCFATURA"      => $params['NRPARCFATURA'],
            "CDTIPOOPERFATURA"  => $params['CDTIPOOPERFATURA'],
            "CDSERIENF"         => $params['CDSERIENF'],
            "CDEMPRESA"         => $params['CDEMPRESA'],
            "CDINSCESTA"        => $params['CDINSCESTA'],
            "CDINSCMUNI"        => $params['CDINSCMUNI'],
        );

        $valor = $this->EscopoFaturamentoFactory->validaDescricaoEscopoDuplicados($paramsValida);

        if ($valor[0]["QTD"] > 0) {
            throw new \Exception("EXISTE");
        }

        $this->connection->beginTransaction();

        try {
            $nrorg = $this->Environment->getNrOrg();
            $cdoperador = $this->Environment->getCdOperador();

            if ($params["NRDESCAGRUPESCOPO"] == null) {
                $nrdescagrupescopo = $this->newCode->getCodeByTableName("FCT_DESCAGRUPESCOPO", $nrorg, 1, 10, true);

                $dataParams = array(
                    "NRDESCAGRUPESCOPO" => $nrdescagrupescopo,
                    "DSNOMEAGRUPAMENTO" => $params["DSNOMEAGRUPAMENTO"],
                    "NRTIPOORCAMENTO"   => $params["NRTIPOORCAMENTO"],
                    "IDATIVO"           => $params["IDATIVO"],
                    "IDTPFATURA"        => $params["IDTPFATURA"],
                    "CDFILIALFATURA"    => $params["CDFILIALFATURA"],
                    "CDEMPRESA"         => $params["CDEMPRESA"],
                    "NRPARCFATURA"      => $params["NRPARCFATURA"],
                    "CDTIPOOPERFATURA"  => $params["CDTIPOOPERFATURA"],
                    "CDSERIENF"         => $params["CDSERIENF"],
                    "CDINSCESTA"        => $params["CDINSCESTA"],
                    "CDINSCMUNI"        => $params["CDINSCMUNI"],
                    "IDPREVCOBRFUT"     => 'N',
                    "NRORG"             => $nrorg,
                    "CDOPERINCLUSAO"      => $cdoperador,
                );

                $this->EscopoFaturamentoFactory->insertDescricaoEscopo($dataParams);
            } else {
               
                $dataParams = array(
                    "NRDESCAGRUPESCOPO" => $params["NRDESCAGRUPESCOPO"],
                    "DSNOMEAGRUPAMENTO" => $params["DSNOMEAGRUPAMENTO"],
                    "IDATIVO"           => $params["IDATIVO"],
                    "IDTPFATURA"        => $params["IDTPFATURA"],
                    "CDFILIALFATURA"    => $params["CDFILIALFATURA"],
                    "CDEMPRESA"         => $params["CDEMPRESA"],
                    "NRPARCFATURA"      => $params["NRPARCFATURA"],
                    "CDTIPOOPERFATURA"  => $params["CDTIPOOPERFATURA"],
                    "CDSERIENF"         => $params["CDSERIENF"],
                    "CDINSCESTA"        => $params["CDINSCESTA"],
                    "CDINSCMUNI"        => $params["CDINSCMUNI"],
                    "IDPREVCOBRFUT"     => 'N',
                    "NRORG"             => $nrorg,
                    "NRORGULTATU"       => $nrorg,
                    "CDOPERULTATU"      => $cdoperador,
                );

                $this->EscopoFaturamentoFactory->updateDescricaoEscopo($dataParams);
            }
            $this->connection->commit();

        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function deleteEscFatDescricaoEscopo($parametros)
    {
        $this->connection->beginTransaction();
        try {
            foreach ($parametros as $parametrodescEsc) {
                $dataParam = array( "NRDESCAGRUPESCOPO" => $parametrodescEsc["NRDESCAGRUPESCOPO"],);
                $this->EscopoFaturamentoFactory->deleteEscFatDescricaoEscopo($dataParam);
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function getRelacAgrupTipoOrcaEscopo($params)
    {
        $params = array(
            "NRTIPOORCAMENTO" => $params['NRTIPOORCAMENTO'],
        );
        return $this->EscopoFaturamentoFactory->getRelacAgrupTipoOrcaEscopo($params);
    }

    public function getModDocFiscParamFat($params)
    {
        $params = array(
            "CDTIPOOPER" => isset($params['CDTIPOOPER'])? $params['CDTIPOOPER']: null,
        );
        return $this->EscopoFaturamentoFactory->getModDocFiscParamFat($params);
    } 
}
