<?php

namespace SM\Service;

use SM\Factory\Sm00005000TabParametrizacoesServicesFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;

class Sm00005000TabParametrizacoesServicesService
{
    protected $connection;
    protected $parametrizacaoFactory;
    protected $environment;
    protected $newCode;

    public function __construct(Sm00005000TabParametrizacoesServicesFactory $parametrizacaoFactory, Environment $environment, NewCode $newCode, EntityManager $entityManager)
    {
        $this->parametrizacaoFactory        = $parametrizacaoFactory;
        $this->environment                  = $environment;
        $this->newCode                      = $newCode;
        $this->connection                   = $entityManager->getConnection();
    }

    public function getParametrosGeral()
    {
        return $this->parametrizacaoFactory->getParametrosGeral();
    }

    public function getNomePerfAcess()
    {
        return $this->parametrizacaoFactory->getNomePerfAcess();       
    }

    public function buscaNovoCodigo($params)
    {
        $this->connection->beginTransaction();
        try {
            $data = $this->parametrizacaoFactory->getParametrosGeral();
            if ($params["novoCodigo"] != null && $data["data"][0]["NRINISEQCODORCA"] != $params["novoCodigo"]) {
                $parametros = array(
                    "CDCONTADOR"    =>  $params["cdContador"],
                    "NRORG"         =>  $this->environment->getNrOrg()
                );
                return $this->parametrizacaoFactory->buscaNovoCodigo($parametros);
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function atualizaNovoCodigo($params)
    {
        $this->connection->beginTransaction();

        try {
            $params = array(
                "NRSEQUENCIAL" => $params["params"]["novoCodigo"],
                "NRORG"        => $this->environment->getNrOrg(),
                "OPERADOR"     => $this->environment->getCdOperador(),
                "CDCONTADOR"   => $params["params"]["cdContador"],
            );
            $this->parametrizacaoFactory->atualizaNovoCodigo($params);
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function insereNovoCodigo($params)
    {
        $this->connection->beginTransaction();

        try {
            $params = array(
                "NRSEQUENCIAL" => $params["novoCodigo"],
                "NRORG"        => $this->environment->getNrOrg(),
                "OPERADOR"     => $this->environment->getCdOperador(),
                "CDCONTADOR"   => $params["cdContador"],
            );
            $this->parametrizacaoFactory->insereNovoCodigo($params);
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function saveperfilacesso($params)
    {
        $this->connection->beginTransaction();
        try {
            $data = $this->parametrizacaoFactory->getParametrosGeral();
            $perfilacessoProposta = empty($data["data"][0]["FCT_ALTPROPCONCPERF"]) ? null : $params["NRPERFILACESSOPROPOSTA"];
            $perfilacessoContrato = empty($data["data"][0]["FCT_ALTCONTRAPROVPERF"]) ? null : $params["NRPERFILACESSOCONTRATO"];
            $nrorg = $data["data"][0]["NRORG"];
            $nrorginclusao = $data["data"][0]["NRORGINCLUSAO"];
            $cdoperinclusao = $data["data"][0]["CDOPERINCLUSAO"];
            $cdoperultatu = $data["data"][0]["CDOPERULTATU"];
            $nrorgultatu = $data["data"][0]["NRORGULTATU"];
            $dtinclusao = $data["data"][0]["DTINCLUSAO"];
            $dtultatu = $data["data"][0]["DTULTATU"];
            $idativo = $data["data"][0]["IDATIVO"];
            $proposta = null;
            $contrato = null;

            $buscaEmPerfisContratoExistentes = $this->buscarperfilacesso($nrorg, 'C');
            $buscaEmPerfisPropostaExistentes = $this->buscarperfilacesso($nrorg, 'P');
            $perfisContratoExistentes = array();
            $perfisPropostaExistentes = array();
    
            foreach ($buscaEmPerfisContratoExistentes["data"] as $removedorarray) {
                array_push($perfisContratoExistentes, $removedorarray["NRPERFILACESSO"]);
            }

            foreach ($buscaEmPerfisPropostaExistentes["data"] as $removedorarray) {
                array_push($perfisPropostaExistentes, $removedorarray["NRPERFILACESSO"]);
            }

            if ($perfilacessoProposta == null) {
                $perfilacessoProposta = array();
            }

            if ($perfilacessoContrato == null) {
                $perfilacessoContrato = array();
            }

            // PROPOSTA se existe algum perfil na row que não existe no banco
            foreach ($perfilacessoProposta as $perfil) {
                if (!in_array($perfil, $perfisPropostaExistentes)) {
                    $proposta = true;
                    $contrato = false;
                    $this->saveperfilacessoFactory($perfil, $nrorg, $nrorginclusao, $cdoperinclusao, $cdoperultatu, $nrorgultatu, $dtinclusao, $dtultatu, $idativo, $proposta, $contrato);
                }
            }

            //PROPOSTA se existe algum perfil no banco que foi removido da row
            foreach ($perfisPropostaExistentes as $perfil) {
                if (!in_array($perfil, $perfilacessoProposta)) {
                    $proposta = true;
                    $contrato = false;
                    $this->deleteperfilacesso($perfil, $proposta, $contrato);
                }
            }

            //CONTRATO se existe algum perfil na row que não existe no banco
            foreach ($perfilacessoContrato as $perfil) {
                if (!in_array($perfil, $perfisContratoExistentes)) {
                    $contrato = true;
                    $proposta = false;
                    $this->saveperfilacessoFactory($perfil, $nrorg, $nrorginclusao, $cdoperinclusao, $cdoperultatu, $nrorgultatu, $dtinclusao, $dtultatu, $idativo, $proposta, $contrato);
                }
            }

            //CONTRATO se existe algum perfil no banco que ofi removido da row
            foreach ($perfisContratoExistentes as $perfil) {
                if (!in_array($perfil, $perfilacessoContrato)) {
                    $contrato = true;
                    $proposta = false;
                    $this->deleteperfilacesso($perfil, $proposta, $contrato);
                }
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function buscarperfilacesso($nrorg, $perfilDerivado)
    {
        $params = array(
            "NRORG" => $nrorg
        );
        if ($perfilDerivado == "C") {
            $result = $this->parametrizacaoFactory->buscarperfilacessoContrato($params);
            return $result;
        }
        if ($perfilDerivado == "P") {
            $result = $this->parametrizacaoFactory->buscarperfilacessoProposta($params);
            return $result;
        }
    }

    public function saveperfilacessoFactory($perfilacesso, $nrorg, $nrorginclusao, $cdoperinclusao, $cdoperultatu, $nrorgultatu, $dtinclusao, $dtultatu, $idativo, $proposta, $contrato)
    {
        $this->connection->beginTransaction();
        try {
            $nraltcontraprovperf = $this->newCode->getCodeByTableName("NRALTCONTRAPROVPERF", $nrorg, 1, 10, true);
            $params = array(
                "NRALTCONTRAPROVPERF"  =>  $nraltcontraprovperf,
                "NRPERFILACESSO"  => $perfilacesso,
                "NRORG"          => $nrorg,
                "NRORGINCLUSAO"  => $nrorginclusao,
                "CDOPERINCLUSAO"  => $cdoperinclusao,
                "CDOPERULTATU"  => $cdoperultatu,
                "NRORGULTATU" => $nrorgultatu,
                "DTINCLUSAO"  => $dtinclusao,
                "DTULTATU"  => $dtultatu,
                "IDATIVO"  =>  $idativo
            );
            if ($proposta) {
                $this->parametrizacaoFactory->saveperfilacessoProposta($params);
            }
            if ($contrato) {
                $this->parametrizacaoFactory->saveperfilacessoContrato($params);
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function deleteperfilacesso($perfilacesso, $proposta, $contrato)
    {
        $this->connection->beginTransaction();
        try {
            $params = array(
                "NRPERFILACESSO"  => $perfilacesso
            );
            if ($proposta) {
                $this->parametrizacaoFactory->deleteperfilacessoProposta($params);
            }
            if ($contrato) {
                $this->parametrizacaoFactory->deleteperfilacessoContrato($params);
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function saveDadosParametrosService($params)
    {
        $this->connection->beginTransaction();
        try {
            $dataParams = array (
                "IDUTILIZAHIEROPER"         => $params["data"]["IDUTILIZAHIEROPER"],
                "IDVALIDACPFRF"             => $params["data"]["IDVALIDACPFRF"],
                "IDOBRIGMODORC"             => $params["data"]["IDOBRIGMODORC"],
                "IDCONVCLIENTE"             => $params["data"]["IDCONVCLIENTE"],
                "NRINISEQCODORCA"           => $params["data"]["NRINISEQCODORCA"],
                "IDALTERAPROPCONC"          => $params["data"]["IDALTERAPROPCONC"],
                "NRINISEQCODCONT"           => $params["data"]["NRINISEQCODCONT"],
                "IDALTERACONTAPROV"         => $params["data"]["IDALTERACONTAPROV"],
                "IDBUSCADTVENCNF"           => $params["data"]["IDBUSCADTVENCNF"],
                "DSEMAILALERTRENOVCONT"     => $params["data"]["DSEMAILALERTRENOVCONT"],
                "IDPERIODALERTRENOVCONT"    => $params["data"]["IDPERIODALERTRENOVCONT"],
                "IDBLOQREAJSEMALIQ"         => $params["data"]["IDBLOQREAJSEMALIQ"],
                "IDBLOQAPURFATSEMREAJ"      => $params["data"]["IDBLOQAPURFATSEMREAJ"],
                "DSHOSTEMAIL"               => $params["data"]["DSHOSTEMAIL"],
                "DSPORTAEMAIL"              => $params["data"]["DSPORTAEMAIL"],
                "DSUSUARIOEMAIL"            => $params["data"]["DSUSUARIOEMAIL"],
                "DSSENHAEMAIL"              => $params["data"]["DSSENHAEMAIL"],
                "IDSMTPSECURE"              => $params["data"]["IDSMTPSECURE"],
                "DSURLINTEGRACAOFM"         => $params["data"]["DSURLINTEGRACAOFM"],
                "DSTOKENCERTSIGN"           => $params["data"]["DSTOKENCERTSIGN"],
                "NMSENDERCERTSIGN"          => $params["data"]["NMSENDERCERTSIGN"],
                "DSEMAILSENDERCERTSIGN"     => $params["data"]["DSEMAILSENDERCERTSIGN"],
                "NRCPFSENDERCERTSIGN"       => $params["data"]["NRCPFSENDERCERTSIGN"],
                "IDPADRAOASSINACERTSIGN"    => $params["data"]["IDPADRAOASSINACERTSIGN"],
                "URLINTEGRACAOERPFILIAL"    => $params["data"]["URLINTEGRACAOERPFILIAL"],
                "NRTIPOLOCALCLIFM"          => $params["data"]["NRTIPOLOCALCLIFM"],
                "NRTIPODOCCERTSIGN"         => $params["data"]["NRTIPODOCCERTSIGN"],
                "URLINTEGRACAOTECFOOD"      => $params["data"]["URLINTEGRACAOTECFOOD"],
            );
            $this->parametrizacaoFactory->saveDadosParametrosService($dataParams);
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
}
