/***********************************************************************|
| CORE VISION                                                           |
|                                                                       |
|                                                                       |
| Nome: ETL_FATORESUF                                                   |
|                                                                       |
| Criao:                                                              |
|    04/09/2006 - Fatima                                                |
|                                                                       |
|  @C:\Web_503\Sql\End_Conv\Oracle\9320_GSCSTPO_ETL_FATORESUF.SQL       |
|                                                                       |
| Alterao:                                                            |
|    02/05/2008 - Raphael Ernani                                        |
|      Alterao do comprimento do campo MEFATO para 2 caracteres       |
|                                                                       |
| Alterao:                                                            |
|    07/05/2008 - Leandro Corra Gonalves                              |
|      Otimizao das rotinas de Despesas, Entrada NF e Sada NF        |
|                                                                       |
| Alterao                                                             |
|    19/05/2008 - Leandro Corra Gonalves                              |
|      Acrscimo da dimenso Data de Processamento	                |
|      								        |
+************************************************************************/

/* EXEC ETL_FATORESUF */

CREATE OR REPLACE PROCEDURE ETL_FATORESUF
                     (P_DTINI      IN DATE,
                      P_DTFINA     IN DATE)
                     
IS
  V_DTFIM           DATE;
  V_DTINI           DATE;
  V_NRSEQFATRES     FATORESUFAT.NRSEQFATRES%TYPE;
  V_DESPESA         NOTAFISCAL.VRNOTAFISC%TYPE;
  V_MOBRA           NOTAFISCAL.VRNOTAFISC%TYPE;
  V_VRFATUR         NOTAFISCAL.VRNOTAFISC%TYPE;
  V_VRIMPOSTO       NOTAFISCAL.VRNOTAFISC%TYPE;
  V_FATLIQ          NOTAFISCAL.VRNOTAFISC%TYPE;
  V_ENTRADANF_LIQ   NOTAFISCAL.VRNOTAFISC%TYPE;
  V_ICMS_ENTRADANF  NOTAFISCAL.VRNOTAFISC%TYPE;
  V_ESTOQUE_INICIAL NOTAFISCAL.VRNOTAFISC%TYPE;
  V_ESTOQUE_FINAL   NOTAFISCAL.VRNOTAFISC%TYPE;
  V_SAIDANF_LIQ     NOTAFISCAL.VRNOTAFISC%TYPE;
  V_ICMS_SAIDANF    NOTAFISCAL.VRNOTAFISC%TYPE;
  V_CONSUMO         NOTAFISCAL.VRNOTAFISC%TYPE;
  V_RESULT          NOTAFISCAL.VRNOTAFISC%TYPE;
  V_DTMES           VARCHAR2(02);
  V_DTANO           VARCHAR2(04);
  V_TIPOFATU        VARCHAR2(30);
 
 
BEGIN
    
    BEGIN
    SELECT NVL(MAX(NRSEQFATRES),0) 
    INTO   V_NRSEQFATRES
    FROM FATORESUFAT;
    EXCEPTION
    WHEN OTHERS THEN
         V_NRSEQFATRES := '0';
    END;
     
     V_DTMES  :=  TO_CHAR(P_DTINI,'MM');
     V_DTANO  :=  TO_CHAR(P_DTINI,'YYYY');

    DELETE FROM FATORESUFAT
    WHERE  (ANFATO > V_DTANO) OR
           (ANFATO = V_DTANO AND MEFATO >= V_DTMES);
    COMMIT;

    V_DTFIM  := LAST_DAY(V_DTINI);  
    V_DTINI  := P_DTINI;
    
    WHILE (V_DTINI  <=  P_DTFINA) LOOP
              
          V_DTFIM := LAST_DAY( V_DTINI);    
          
          FOR C_FILIAL IN
             (SELECT F.CDFILIAL, CDEMPRESA, 
                     F.CDAREAGERE      AS CDAREAGERE,
                     F.CDGOPERAC       AS CDGOPERAC,
                     F.CDSUPERVISOR  AS CDSUPERVISOR,
                     F.CDSEGMER          AS CDSEGMER,
                     P.CDTIPOFATUR     AS CDTIPOFATUR,
                     NVL(F.QTFUNCFILI,1)     AS QTFUNCFILI,
                     P.DTPROCESSA
              FROM   FILIAL F,  PARAMFILIAL P
              WHERE  (F.CDFILIAL = P.CDFILIAL) 
              ) 
          LOOP   

/* Faturamento e Impostos  */                

              SELECT NVL(SUM(A.VALORBRUTO),0), NVL(SUM(A.VRIMPOSTO),0) 
              INTO V_VRFATUR,  V_VRIMPOSTO
              FROM
                   (SELECT CDFILIAL, NRLANCTONF, DTENTRSAID,
                         SUM(N.VRNOTAFISC) AS VALORBRUTO,
                         SUM(VALOR_IMPOSTO_NF(CDFILIAL, NRLANCTONF, '00','I')) +
                         SUM(VALOR_IMPOSTO_NF(CDFILIAL, NRLANCTONF, '31','I')) +  
                         SUM(VALOR_IMPOSTO_NF(CDFILIAL, NRLANCTONF, '50','I')) +
                         SUM(VALOR_IMPOSTO_NF(CDFILIAL, NRLANCTONF, '60','I')) AS VRIMPOSTO
                   FROM 
                         NOTAFISCAL N
                   WHERE 
                             (N.CDFILIEMIT       =    C_FILIAL.CDFILIAL)                        
                         AND (N.DTENTRSAID BETWEEN V_DTINI AND V_DTFIM)            
                         AND (N.IDENTRSAIDOP     = 'S')                                                         
                         AND ((N.IDSTATUS    IN ('A','E')) or (N.IDSISTEMA = 'PDV'))           
                         AND (N.CDTIPOOPER IN ('01','17'))                                                    
                         AND (N.IDSISTEMA NOT IN ('PAG','REC','FIS','CMP'))                   
                   GROUP BY  CDFILIAL, NRLANCTONF, DTENTRSAID
                     ) A ;
             
/* Despesas  */        

             SELECT SUM(VALOR)
               INTO V_DESPESA
               FROM
                  ((SELECT SUM(((NVL(I.VRTOTAITEM,0) - NVL(I.VRDESCITEM,0)) - ((NVL(I.VRTOTAITEM,0) / NVL(N.VRPRODUTOS,0)) 
                                 * NVL(T.VRITEMENCA,0)))) VALOR
                      FROM
                          NOTAFISCAL N,
                          ITEMPRODUTO I,
                          PRODUTO P,
                          ITEMENCARGO T
                     WHERE (N.IDENTRSAIDOP  = 'E')                                                
                       AND (N.CDFILIRECE    = C_FILIAL.CDFILIAL)                      
                       AND (N.CDTIPOOPER IN ('01','10'))                                                                     
                       AND (N.DTENTRSAID BETWEEN V_DTINI AND V_DTFIM)                            
                       AND (N.IDSTATUS  <> 'C')                                                                                 
                       AND (N.CDFILIAL = I.CDFILIAL)                                                                       
                       AND (N.NRLANCTONF = I.NRLANCTONF)                                                    
                       AND (N.CDFILIAL     = T.CDFILIAL(+))                                                              
                       AND (N.NRLANCTONF   = T.NRLANCTONF(+))                                             
                       AND (T.CDENCARGO(+) = '03')                                                                           
                       AND ((P.CDARVPROD LIKE 'D%') OR (P.CDARVPROD LIKE 'P%'))
                       AND (I.CDPRODUTO = P.CDPRODUTO) )
          UNION ALL
               SELECT NVL(SUM(A.VRREALNESTO),0) AS VALOR
                 FROM CUSTNESTO A,
                      PRODUTO P
                WHERE (A.CDFILIAL   = C_FILIAL.CDFILIAL)                                         
                  AND (A.DTCUSTNESTO BETWEEN V_DTINI AND V_DTFIM)                     
                  AND (A.CDPRODUTO  = P.CDPRODUTO)                                                
                  AND ((P.CDARVPROD LIKE 'D%') OR (P.CDARVPROD LIKE 'P%')));
                  
/* Mao Obra */ 

              SELECT NVL(SUM(A.VRREALNESTO),0) 
              INTO V_MOBRA
              FROM CUSTNESTO A,
                   PRODUTO P
              WHERE
                       (A.CDFILIAL = C_FILIAL.CDFILIAL)                                                                      
                   AND (A.DTCUSTNESTO BETWEEN V_DTINI AND V_DTFIM)                                   
                   AND (A.CDPRODUTO = P.CDPRODUTO)                                                                        
                   AND ((SUBSTR(CDARVPROD,1,1) <> 'D') AND (SUBSTR(CDARVPROD,1,1) <> 'P')  
                   AND (SUBSTR(CDARVPROD,1,1) <> 'F') AND (SUBSTR(CDARVPROD,1,1) <> 'O')) ;

/* Estoque inicial */ 

            SELECT NVL(SUM(DECODE(SUBSTR(A.CDARVPROD,1,1),'D',0, P.VRESTOQDIA)),0)
               INTO V_ESTOQUE_INICIAL
               FROM PRODUTO A, POSESTDIA P
              WHERE    (P.CDFILIAL        = C_FILIAL.CDFILIAL)             
                   AND (P.CDPRODUTO  = A.CDPRODUTO)                     
                   AND (A.CDPRODUTO  = A.CDPRODESTO)                   
                   AND (P.DTPOSIESTQ   = (SELECT MAX(DTPOSIESTQ)
                                            FROM POSESTDIA C
                                            WHERE (C.CDFILIAL     = C_FILIAL.CDFILIAL)       
                                              AND (C.CDPRODUTO    = P.CDPRODUTO)             
                                              AND (C.CDALMOXARIFE = P.CDALMOXARIFE)       
                                              AND (C.CDLOCALESTOQ = P.CDLOCALESTOQ)      
                                              AND (C.NRLOTEESTQ   = P.NRLOTEESTQ)            
                                              AND (C.NRSUBLOTE    = P.NRSUBLOTE)               
                                              AND (C.DTPOSIESTQ   <= V_DTINI)));

/* Estoque Final */ 

          SELECT NVL(SUM(DECODE(SUBSTR(A.CDARVPROD,1,1),'D',0, P.VRESTOQDIA)),0) 
            INTO V_ESTOQUE_FINAL
            FROM PRODUTO A, POSESTDIA P
           WHERE (P.CDFILIAL = C_FILIAL.CDFILIAL) 
            AND (P.CDPRODUTO  = A.CDPRODUTO)                    
            AND (A.CDPRODUTO  = A.CDPRODESTO)                  
            AND (P.DTPOSIESTQ = (SELECT MAX(DTPOSIESTQ)
                                   FROM POSESTDIA C
                                  WHERE (C.CDFILIAL       = C_FILIAL.CDFILIAL)                                
                                    AND (C.CDPRODUTO      = P.CDPRODUTO)           
                                    AND (C.CDALMOXARIFE   = P.CDALMOXARIFE)       
                                    AND (C.CDLOCALESTOQ   = P.CDLOCALESTOQ)      
                                    AND (C.NRLOTEESTQ     = P.NRLOTEESTQ)            
                                    AND (C.NRSUBLOTE      = P.NRSUBLOTE)              
                                    AND (C.DTPOSIESTQ     <= V_DTFIM + 1 )));
 /* Entrada NF */ 

             SELECT NVL(SUM(DECODE(SUBSTR(CDARVPROD,1,1),'D','0', VRBRUTITEM)),0),NVL(SUM((DECODE(SUBSTR(CDARVPROD,1,1),'D','0', DECODE(T.CDFISCALIT,'1',T.VRIMPOSTOIT,0.0)))),0) 
               INTO V_ENTRADANF_LIQ, V_ICMS_ENTRADANF 
               FROM NOTAFISCAL N, ITEMPRODUTO I, ITEMPROIMPOS T, PRODUTO P
              WHERE (N.CDFILIRECE      = C_FILIAL.CDFILIAL)                                              
                AND (N.IDENTRSAIDOP    = 'E')                                                                             
                AND (INSTR('_01_02_04_11_16_',N.CDTIPOOPER) <> 0)                                              
                AND (N.IDSTATUS  <>  'C')                                                                         
                AND (INSTR('_PAG_REC_FIS_CMP_',N.IDSISTEMA) = 0)
                AND (N.DTENTRSAID BETWEEN V_DTINI AND V_DTFIM)                                       
                AND (N.CDFILIAL       = I.CDFILIAL)                                                                     
                AND (N.NRLANCTONF     = I.NRLANCTONF)                                                           
                AND (I.CDFILIAL       = T.CDFILIAL(+))                                                              
                AND (I.NRLANCTONF     = T.NRLANCTONF(+))                                                     
                AND (I.NRSEQITPROD    = T.NRSEQITPROD(+))                                                   
                AND (T.NRITPROIMPOS(+) = '1')                                                                                   
                AND (T.CDIMPOSTO(+)    = '00')                                                                     
                AND (I.CDPRODUTO       = P.CDPRODUTO) ;       
            
/* Saida NF */ 

              SELECT NVL(SUM(DECODE(SUBSTR(CDARVPROD,1,1),'D','0', VRBRUTITEM)),0),NVL(SUM((DECODE(SUBSTR(CDARVPROD,1,1),'D','0', DECODE(T.CDFISCALIT,'1',T.VRIMPOSTOIT,0.0)))),0) 
                INTO V_SAIDANF_LIQ, V_ICMS_SAIDANF 
                FROM NOTAFISCAL N, ITEMPRODUTO I, ITEMPROIMPOS T, PRODUTO P
               WHERE (N.CDFILIEMIT = C_FILIAL.CDFILIAL)                               
                 AND (N.IDENTRSAIDOP = 'S')                                                       
                 AND (N.CDTIPOOPER IN ('02','06'))                                               
                 AND (I.CDPRODUTO  = P.CDPRODUTO)                                            
                 AND (N.IDSTATUS    <> 'C')                                                          
                 AND (INSTR('_PAG_REC_FIS_CMP_',N.IDSISTEMA) = 0)  
                 AND (N.DTENTRSAID BETWEEN  V_DTINI AND V_DTFIM)      
                 AND (N.CDFILIAL = I.CDFILIAL)                                                  
                 AND (N.NRLANCTONF = I.NRLANCTONF)                                 
                 AND (I.CDFILIAL = T.CDFILIAL(+))                                                
                 AND (I.NRLANCTONF = T.NRLANCTONF(+))                             
                 AND (I.NRSEQITPROD = T.NRSEQITPROD(+))                              
                 AND (T.CDIMPOSTO(+)   = '00')                                                       
                 AND (T.NRITPROIMPOS(+) = '1') ;     

              
              V_CONSUMO  := (V_ESTOQUE_INICIAL + (V_ENTRADANF_LIQ - V_ICMS_ENTRADANF) -  
                            (V_SAIDANF_LIQ - V_ICMS_SAIDANF) - V_ESTOQUE_FINAL);
                                                   
              V_FATLIQ  :=  V_VRFATUR - V_VRIMPOSTO;
                    
              V_RESULT   :=  (V_FATLIQ - V_DESPESA - V_MOBRA - V_CONSUMO );

              V_DTMES  :=  TO_CHAR(V_DTFIM,'MM');
              V_DTANO  :=  TO_CHAR(V_DTFIM,'YYYY');
              
              V_TIPOFATU :=
                  CASE  C_FILIAL.CDTIPOFATUR
                      WHEN '001'  THEN   'Gesto Direta'
                      WHEN '003'  THEN   'Mandato'
                      WHEN '004'  THEN   'Refeies e Servicos'
                      WHEN '005'  THEN   'Taxa Fixa'
                      WHEN '006'  THEN   'Gesto Hospitalar'
                      WHEN '007'  THEN   'Produtos e Servios'
                      WHEN '008'  THEN   'Produtos e Servios'
                      ELSE 'Sem Tipo Faturamento'
                  END;
                   
              IF  V_FATLIQ = 0 THEN
                  V_FATLIQ := 1;
              END IF;
                   
              IF  C_FILIAL.QTFUNCFILI = 0 THEN
                  C_FILIAL.QTFUNCFILI := 1;
              END IF; 
              
                                                  
              IF  V_VRFATUR > 0 OR V_DESPESA > 0 OR V_MOBRA >0 OR V_CONSUMO > 0 THEN
                   V_NRSEQFATRES := V_NRSEQFATRES + 1;
                   INSERT INTO FATORESUFAT
                        (CDEMPRESA,  CDFILIAL,  CDAREAGERE ,  CDGOPERAC,  
                         CDSUPERVISOR, CDSEGMER,  CDTIPOFATURE, CDSERVFILI ,  
                         ANFATO,  MEFATO , VRFATOFATU, VRFATOIMPO, 
                         QTFATOCOME, QTFATOCOMF,  VRFATOCONS, VRMAOOBRA, 
                         VRFATOMATPR, VRFATODESPE, VRFATOFUNC, VRFATOLIQNF, 
                         VRFATORESU, QTFATOFUNC, VRFATOMARG, VRPERMDOFAT, 
                         VRCUSTOMDO, NRSEQFATRES,DSDTPROCFATOFAT)
                  VALUES
                        (C_FILIAL.CDEMPRESA, C_FILIAL.CDFILIAL, C_FILIAL.CDAREAGERE, C_FILIAL.CDGOPERAC,
                         C_FILIAL.CDSUPERVISOR, C_FILIAL.CDSEGMER, V_TIPOFATU, NULL,
                         V_DTANO, V_DTMES, V_VRFATUR, V_VRIMPOSTO,
                         0, 0, V_CONSUMO, V_MOBRA,
                         0, V_DESPESA, (V_FATLIQ / C_FILIAL.QTFUNCFILI), V_FATLIQ,  
                         V_RESULT, C_FILIAL.QTFUNCFILI, ((V_RESULT / V_FATLIQ) *100), ((V_MOBRA / V_FATLIQ) *100),
                         (V_MOBRA / C_FILIAL.QTFUNCFILI), LPAD(V_NRSEQFATRES,12,0),TO_CHAR(C_FILIAL.DTPROCESSA,'DD/MM/YYYY'));

              END IF;


         END LOOP;

         COMMIT;

          V_DTINI := ADD_MONTHS(V_DTINI,1);

    END LOOP;
END;
/


-- VERSAO --
BEGIN ATUVEROBJ('ETL_FATORESUF','5.05.019'); END;
/
