import { Utilities } from '@/util/Utilities';
import {
    Form, LoadingService, Modal, ModalService, TextInput, DialogService, Button,
    Tabs,
    SelectMultiple,
} from '@zeedhi/common';
import { Http, Metadata, IEventParam } from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';

export class HabilidadePessoalController {
    private addHabilidadePessoalModal!: Modal;

    private addPessoaModal!: Modal;

    private tekgridHabilidadePessoal: boolean = true;

    private currentRow: any = {};

    get framehabilidadePessoalvisible() {
        return !this.tekgridHabilidadePessoal;
    }
 
    get tituloFrameHabilidadePessoal() {
        return this.currentRow.NMHABILITSERV;
    }

    public abrirFrameHabilidadePessoal({ row }: any) {
        this.tekgridHabilidadePessoal = false;
        this.currentRow = row;
        this.carregarDadosHabilidadePessoal();
    }

    public backTekGridHabilidadePessoal() {
        this.tekgridHabilidadePessoal = true;
        this.ativaTabDescricao();
    }

    public async exibirModalAddHabilidadePessoal() {
        this.addHabilidadePessoalModal = await ModalService.createFromJson('habilidadePessoal/Modal/modal_adicionar_habilidadePessoal', true);
        this.addHabilidadePessoalModal.show();
    }

    public fecharModalAddHabilidadePessoal() {
        this.addHabilidadePessoalModal.hide();
        this.addHabilidadePessoalModal?.destroy();
    }

    public async abrirModalAddHabilidadePessoal() {
        await this.exibirModalAddHabilidadePessoal();
    }

    public async exibirModalAddPessoa() {
        this.addPessoaModal = await ModalService.createFromJson('habilidadePessoal/Modal/modal_adicionar_pessoas', true);
        this.addPessoaModal.show();
    }

    public fecharModalAddPessoa() {
        this.addPessoaModal.hide();
        this.addPessoaModal?.destroy();
    }

    public async abrirModalAddPessoa() {
        await this.exibirModalAddPessoa();
    }

    public ativaTabDescricao() {
        const setOfTabs = Metadata.getInstance<Tabs>('tabs-habilidadepessoal');
        setOfTabs.activeTab = 0;
    }

    public loadTabsHabilidadePessoal() {
        const setOfTabs = Metadata.getInstance<Tabs>('tabs-habilidadepessoal');
        
        if (setOfTabs.tabs[setOfTabs.activeTab].name === 'tab_habilidadePessoa') {
            this.carregarDadosHabilidadePessoal();
        } else if (setOfTabs.tabs[setOfTabs.activeTab].name === 'tab_pessoas') {
            this.setFilterPessoas();
        }
    }

    public setFilterPessoas() {
        const cdhabilitserv = this.currentRow.CDHABILITSERV;
        try {
            const gridpessoa = Metadata.getInstance<TekGrid>('tek-grid-pessoas');
            gridpessoa.datasource.setFilter({
                CDHABILITSERV: cdhabilitserv,
            });
        } catch (error) {
            // console.log(error);
        }
    }

    public salvarModalAddHabilidadePessoal() {
        const gridHabilidadePessoal = Metadata.getInstance<TekGrid>('tek-grid-pessoas');
        const formulario = Metadata.getInstance<Form>('form_modal_adicionar_habilidade_pessoal');
        
        if (Utilities.validateForm(formulario)) {
            const params = {
                NMHABILITSERV: Metadata.getInstance<TextInput>('FORMHABILIDADE-NMHABILITSERV').value,
            };
            LoadingService.show();
            Http.post('/adicionarHabilidadePessoal', params).then(() => {
                this.fecharModalAddHabilidadePessoal();
                gridHabilidadePessoal.datasource.get();
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                console.warn(error);
            }).finally(() => {
                LoadingService.hide();
            });
            console.warn(gridHabilidadePessoal);
        } else {
            Utilities.showAlertCamposObrigatorios();
        }
    }

    public carregarDadosHabilidadePessoal() {
        Metadata.getInstance<TextInput>('form-CDHABILITSERV').value = this.currentRow.CDHABILITSERV;
        Metadata.getInstance<TextInput>('form-NMHABILITSERV').value = this.currentRow.NMHABILITSERV;
    }

    public carregarDadosPessoa() {
        Metadata.getInstance<TextInput>('form-CDHABILITSERV').value = this.currentRow.CDHABILITSERV;
        Metadata.getInstance<TextInput>('form-NMHABILITSERV').value = this.currentRow.NMHABILITSERV;
    }

    showDialogWithButtonsGrid({ event }: any) {
        event.preventDefault();

        DialogService.show({
            name: 'dialog-deletar-habilidade-pessoal',
            type: 'warning',
            title: 'Excluir Registro',
            text: 'Deseja realmente excluir todos os registros selecionados?',
            buttons: [
                {
                    name: 'NAO',
                    component: 'ZdButton',
                    label: 'Não',
                    outline: true,
                    events: {
                        click: () => {
                            DialogService.hide();
                        },
                    },
                },
                {
                    name: 'SIM',
                    component: 'ZdButton',
                    label: 'Sim',
                    events: {
                        click: () => {
                            this.deleteHabilidadePessoal();
                            DialogService.hide();
                        },
                    },
                },
            ],
        });
    }

    public deleteHabilidadePessoal() {
        const grid = Metadata.getInstance<TekGrid>('basic-tek-grid-habilidadePessoal');
        LoadingService.show();
        Http.post('/deleteHabilidadePessoal', grid.selectedRows)
            .then(() => {
                grid.selectedRows = [];
                grid.cancelEditedRows();
                grid.reload();
                Utilities.showSuccessMessage();
            }).catch((error) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
    }

    public saveTabHabilidadePessoal({ event }: IEventParam<Button>) {
        event!.preventDefault();
        const grid = Metadata.getInstance<TekGrid>('basic-tek-grid-habilidadePessoal');
        const form = Metadata.getInstance<Form>('formTabHabilidadePessoal');

        if (Utilities.validateForm(form)) {
            LoadingService.show();
            const codigo = Metadata.getInstance<TextInput>('form-CDHABILITSERV').value;
            const descricao = Metadata.getInstance<TextInput>('form-NMHABILITSERV').value;
            Http.post('/saveTabHabilidadePessoal', {
                CDHABILITSERV: codigo,
                NMHABILITSERV: descricao,
            }).then(() => {
                grid.datasource.get();
                this.backTekGridHabilidadePessoal();
                Utilities.showSuccessMessage();
            }).catch((error) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }

    public saveModalAddPessoa({ event }: IEventParam<Button>) {
        event!.preventDefault();
        const grid = Metadata.getInstance<TekGrid>('tek-grid-pessoas');
        const form = Metadata.getInstance<Form>('form_modal_adicionar_pessoa');

        if (Utilities.validateForm(form)) {
            LoadingService.show();
            const pessoas = Metadata.getInstance<SelectMultiple>('FORM-PESSOA-NRPARCNEGOCIO').selectValue;
            Http.post('/saveModalAddPessoa', {
                PESSOAS: pessoas,
                CDHABILITSERV: this.currentRow.CDHABILITSERV,
            }).then(() => {
                grid.datasource.get();
                this.fecharModalAddPessoa();
                Utilities.showSuccessMessage();
            }).catch((error) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }

    showDialogWithButtonsPessoas({ event }: any) {
        event.preventDefault();

        DialogService.show({
            name: 'dialog-deletar-pessoas',
            type: 'warning',
            title: 'Excluir Registro',
            text: 'Deseja realmente excluir todos os registros selecionados?',
            buttons: [
                {
                    name: 'NAO',
                    component: 'ZdButton',
                    label: 'Não',
                    outline: true,
                    events: {
                        click: () => {
                            DialogService.hide();
                        },
                    },
                },
                {
                    name: 'SIM',
                    component: 'ZdButton',
                    label: 'Sim',
                    events: {
                        click: () => {
                            this.deletePessoas();
                            DialogService.hide();
                        },
                    },
                },
            ],
        });
    }

    public deletePessoas() {
        const grid = Metadata.getInstance<TekGrid>('tek-grid-pessoas');
        LoadingService.show();
        Http.post('/deletePessoas', grid.selectedRows)
            .then(() => {
                grid.selectedRows = [];
                grid.cancelEditedRows();
                grid.reload();
                Utilities.showSuccessMessage();
            }).catch((error) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
    }
}
